# Locate Xalan-C include paths and libraries
# Written by Alexander Tsvyashchenko for DictD++ package

# This module defines
# XALANC_INCLUDE_DIR, XALANC_LIBRARY, XALANMSG_LIBRARY, XALANC_FOUND.

SET(XALANC_DIR_MESSAGE "Set the environment variable XALANCROOT, or XALANC_INCLUDE_DIR & XALANC_LIBRARIES cmake cache entries")

FIND_PATH(XALANC_INCLUDE_DIR xalanc/Include/XalanVersion.hpp
  "[HKEY_CURRENT_USER\\software\\xalanc\\src]"
  "[HKEY_CURRENT_USER\\xalanc\\src]"
  $ENV{XALANCROOT}/src
  /usr/local/include
  /usr/include
)

FIND_LIBRARY(XALANC_LIBRARY
  NAMES 
    xalan-c
  PATHS
    "[HKEY_CURRENT_USER\\software\\xalanc\\lib]"
    "[HKEY_CURRENT_USER\\xalanc\\lib]"
    $ENV{XALANCROOT}/lib
    /usr/local/lib
    /usr/lib
)

FIND_LIBRARY(XALANMSG_LIBRARY
  NAMES
      xalanMsg
  PATHS
    "[HKEY_CURRENT_USER\\software\\xalanc\\lib]"
    "[HKEY_CURRENT_USER\\xalanc\\lib]"
    $ENV{XALANCROOT}/lib
    /usr/local/lib
    /usr/lib
)

# if the include a the library are found then we have it
IF(XALANC_INCLUDE_DIR)
  IF(XALANC_LIBRARY)
    IF(XALANMSG_LIBRARY)
      SET(XALANC_FOUND "YES" )
    ENDIF(XALANMSG_LIBRARY)
  ENDIF(XALANC_LIBRARY)
ENDIF(XALANC_INCLUDE_DIR)


IF(NOT XALANC_FOUND)
  IF(NOT XALANC_FIND_QUIETLY)
    MESSAGE(STATUS "XalanC was not found. ${XALANC_DIR_MESSAGE}")
  ELSE(NOT XALANC_FIND_QUIETLY)
    IF(XALANC_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "XalanC was not found. ${XALANC_DIR_MESSAGE}")
    ENDIF(XALANC_FIND_REQUIRED)
  ENDIF(NOT XALANC_FIND_QUIETLY)
ENDIF(NOT XALANC_FOUND)

