/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

// dictutil++.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include "resort_utils.h"
//--------------------------------------------------------------------
void PrintUsage()
  {
  std::cout << "Usage:\n\n"
    "dictutil++ resort [options]      To resort index\n"
    "dictutil++ create [options]      To create index\n"
    "\nwhere options are:\n"
    "-ii, --index <file>              Specifies input index\n"
    "-oi, --output-index <file>       Specifies result index\n"
    "-ie, --input-encoding <enc>      Specifies input encoding\n"
    "-oe, --output-encoding <enc>     Specifies output encoding\n"
    "-l, --locale <locale>            Specifies locale used to sort index\n"
    "-d, --dictionary-hd <dict-file>  Specifies input dictionary file in HD format\n"
    "-x, --dictionary-xml <dict-file> Specifies input dictionary file in XML format\n"
    "-n, --numeric                    Assume input index is in numeric format\n"
    "-h, --help                       This message\n";
  }
//--------------------------------------------------------------------
int _tmain(int argc, _TCHAR* argv[])
  {
  if (argc < 4)
    {
    PrintUsage();
    exit(1);
    }

  char *index_name = NULL, *out_index_name = NULL, *inp_enc = NULL, *out_enc = NULL,
       *locale = NULL, *dict_hd = NULL, *dict_xml = NULL, *fl_numeric = NULL, *help = NULL;

  struct Options
    {
    char** p_str;
    char *short_opt, *long_opt;
    char* descr;
    }
  options[] = {
    {&index_name, "-ii", "--index", "input index name"},
    {&out_index_name, "-oi", "--output-index", "output index name"},
    {&inp_enc, "-ie", "--input-encoding", "input encoding name"},
    {&out_enc, "-oe", "--output-encoding", "output encoding name"},
    {&locale, "-l", "--locale", "locale name"},
    {&dict_hd, "-d", "--dictionary-hd", "HD dictionary name"},
    {&dict_xml, "-x", "--dictionary-xml", "XML dictionary name"},
    {&fl_numeric, "-n", "--numeric", NULL},
    {&help, "-h", "--help", NULL},
    {NULL, NULL, NULL, NULL}};

  for (int i = 2; i < argc; )
    {
    ulong i1 = 0;
    for (; options[i1].p_str; ++i1)
      if (!strcmp(argv[i], options[i1].short_opt) ||
          !strcmp(argv[i], options[i1].long_opt))
        {
        if (options[i1].descr)
          {
          if (i == argc - 1)
            {
            std::cerr << "Command line arguments error: " << options[i1].descr << " expected" << std::endl;
            exit(1);
            }
          *options[i1].p_str = argv[i + 1];
          i += 2;
          break;
          }
        else
          {
          *options[i1].p_str = (char*)1;
          ++i;
          break;
          }
        }

    if (!options[i1].p_str)
      {
      PrintUsage();
      exit(2);
      }
    }

  if (!strcmp(argv[1], "resort"))
    {
    if (!index_name || !out_index_name || !locale)
      {
      std::cerr << "Not all arguments for \"resort\" command were specified" << std::endl;
      exit(1);
      }

    if (!inp_enc)
      {
      std::cout << "Input encoding was not specified, will use UTF-8 by default" << std::endl;
      inp_enc = "UTF-8";
      }

    if (!out_enc)
      {
      std::cout << "Output encoding was not specified, will use UTF-8 by default" << std::endl;
      out_enc = "UTF-8";
      }

    ResortIndex(index_name, inp_enc, out_index_name, locale, out_enc, fl_numeric);
    }
  else if (!strcmp(argv[1], "create"))
    {
    if (!out_index_name || !locale || !dict_hd && !dict_xml)
      {
      std::cerr << "Not all arguments for \"create\" command were specified" << std::endl;
      exit(1);
      }

    if (!inp_enc && !dict_xml)
      {
      std::cout << "Input encoding was not specified, will use UTF-8 by default" << std::endl;
      inp_enc = "UTF-8";
      }

    if (!out_enc)
      {
      std::cout << "Output encoding was not specified, will use UTF-8 by default" << std::endl;
      out_enc = "UTF-8";
      }

    if (dict_hd)
      IndexFromHD(dict_hd, inp_enc, out_index_name, locale, out_enc);
    else
      {
      if (inp_enc)
        IndexFromXML(dict_xml, inp_enc, out_index_name, locale, out_enc);
      else
        IndexFromXML(dict_xml, "", out_index_name, locale, out_enc);
      }
    }
  else
    {
    PrintUsage();
    exit(2);
    }
  
  return 0;
  }
//--------------------------------------------------------------------
