/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef UTILS_H
#define UTILS_H

//------------------------------------------------------------------------------
class IndexResorter
  {
  public:
    IndexResorter(const std::string&i_locale);

    ~IndexResorter();

    void AddKeyword(const UnicodeString& i_keyword, index_type i_start,
                    index_type i_length);

    void SortAndWrite(std::ostream& os, const std::string& i_out_enc);

  private:
    struct KeyInfo
      {
      UnicodeString m_keyword;
      index_type m_start, m_length;

      KeyInfo(const UnicodeString& i_keyword, index_type i_start,
              index_type i_length):
        m_keyword(i_keyword), m_start(i_start), m_length(i_length) {}
      };

    struct SortStructure
      {
      CollationKey m_key;
      const KeyInfo* mp_info;

      SortStructure(const CollationKey& i_key, const KeyInfo* ip_info):
        m_key(i_key), mp_info(ip_info) {}
      };

    struct SortStructureRefsComparator
      {
      bool operator () (const SortStructure*& ip_struct0, const SortStructure*& ip_struct1) const
        {
        return ip_struct0 -> m_key.compareTo(ip_struct1 -> m_key) == Collator::LESS;
        }
      };

  std::deque<KeyInfo> m_info;
  std::deque<SortStructure> m_keys;

  Collator* mp_collator;
  };
//------------------------------------------------------------------------------

// Here are some helper functions to prepare or correct data
// to use them together with DictD++
//------------------------------------------------------------------------------
// This function will read dictionary in %h-%d format and construct the
// appropriate index file
void IndexFromHD(const std::string& i_dict_name, const std::string& i_input_enc,
                 const std::string& i_out_index_name, const std::string& i_locale,
                 const std::string& i_output_enc);
//------------------------------------------------------------------------------
void IndexFromXML(const std::string& i_dict_name, const std::string& i_input_enc,
                  const std::string& i_out_index_name, const std::string& i_locale,
                  const std::string& i_output_enc);
//------------------------------------------------------------------------------
// This function will read input index in "input" encoding and create resulting index
// that is sorted according to given locale and has "output" encoding
void ResortIndex(const std::string& i_index_name, const std::string& i_input_enc,
                 const std::string& i_res_index_name, const std::string& i_locale,
                 const std::string& i_output_enc, bool i_fl_numeric = false);
//------------------------------------------------------------------------------

#endif // UTILS_H