/*****************************************************************************
 *
 * DictD++, Copyright (C) 2007 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef DICTCLIENTPP_H
#define DICTCLIENTPP_H

//-----------------------------------------------------------------------------
#if defined(_MSC_VER)
#  if !defined(_WIN32_WINNT)
#    define _WIN32_WINNT 0x0500
#  endif
#  if _MSC_VER >= 1400
#    if !defined(_CRT_SECURE_NO_DEPRECATE)
#      define _CRT_SECURE_NO_DEPRECATE
#    endif
#    if !defined(NOMINMAX)
#      define NOMINMAX
#    endif
#  endif
#  pragma warning (disable: 4267 4996)
#endif
//-----------------------------------------------------------------------------
#include <string>
#include <vector>
#include <deque>
//-----------------------------------------------------------------------------
/// This class provides the functionality to build simple DICT protocol client.
/// Error-handling is quite primitive - functions just throw std::runtime_error
/// with error text description, no exceptions from boost::asio are catched,
/// so do that at calling level.
///
/// If you need to authenticate to use some of the dictionaries,
/// call Auth() right after DictClient() object creation, so that
/// all subsequent queries are done from the authorized user account.
///
/// No results are cached - each query is performed as many times as
/// you call it, so it's the best to cache things like the list of
/// available databases on calling side.
///
/// Only synchronous queries are supported.
//-----------------------------------------------------------------------------
class DictClient
  {
  public:
    typedef std::pair<std::string, std::string> PairItem;
    typedef PairItem MatchItem; ///< dictionary, keyword
    typedef PairItem DBItem; ///< db name, db description
    typedef PairItem StratItem; ///< strat name, strat description

    struct DefinitionItem
      {
      std::string m_word, m_dict_name, m_dict_descr, m_definition;
      };

    typedef std::deque<PairItem> PairItems;
    typedef PairItems Matches;
    typedef PairItems DBs;
    typedef PairItems Strats;

    typedef std::deque<DefinitionItem> Definitions;

  public:
    DictClient(const std::string& server_name, const std::string& service_name);

    ~DictClient();

    void Auth(const std::string& user_name, const std::string& password);

    void Match(Matches& matches, const std::string& db_name,
               const std::string& strat_name, const std::string& word);

    void Define(Definitions& definitions, const std::string& db_name,
                const std::string& word);

    /// Returns list of available dictionaries (AKA databases)
    void GetDBs(DBs& dbs);

    /// Returns list of available strategies
    void GetStrats(Strats& strats);

  private:
    struct DictClientImpl;
    DictClientImpl* mp_impl;

  private:
    void ProcessCommandCommon(PairItems& items, const std::string& command, unsigned not_found_code, unsigned success_code);
  };
//-----------------------------------------------------------------------------

#endif // DICTCLIENTPP_H