/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef ABSTRACT_CONTROLLER_H
#define ABSTRACT_CONTROLLER_H

//------------------------------------------------------------------------------
// This is an interface for controller classes that
// are responsible for the control of query execution;
// that is, classes that support this interface should answer
// IsNextMatchAllowed() and IsNextDefineAllowed() depending on
// factors that are specific for the controller implementation (f.e.
// it may depend on whether server is shutting down or not,
// on maximal number of results allowed, on maximal searching time etc)
//
class AbstractController
  {
  public:
    // This function should be called when matching is strarting
    virtual void StartMatch() = 0;

    // This function should be called when defining is strarting
    virtual void StartDefine() = 0;

    // This function should be called when there is one more item that
    // was matched
    virtual void OneMoreMatch() = 0;

    // This function should be called when there is one more item that
    // was defined
    virtual void OneMoreDefine() = 0;

    // This function should be called to test whether it is allowed
    // to continue matching or not
    virtual bool IsMatchAllowed() = 0;

    // This function should be called to test whether it is allowed
    // to continue defining or not
    virtual bool IsDefineAllowed() = 0;

    // This function returns true if matching process was interrupted
    // due to some reasons
    virtual bool WasMatchInterrupted() = 0;

    // This function returns true if defining process was interrupted
    // due to some reasons
    virtual bool WasDefineInterrupted() = 0;
  };
//------------------------------------------------------------------------------
#endif // ABSTRACT_CONTROLLER_H

