/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef ABSTRACT_INDEX_H
#define ABSTRACT_INDEX_H

#include "abstract_position.h"

//------------------------------------------------------------------------------
// This is an exception class used to report errors
// for all Indices.
class IndexException: public GeneralException
  {
  public:
    IndexException(const std::string& i_err_id,
                   const std::string& i_msg):
      GeneralException("Index", i_err_id, i_msg) {}
  };
//------------------------------------------------------------------------------
// This is an interface for all Indices.
//
// Index model assumes a number of keywords that are sorted
// in some way and each keyword corresponds to position of
// this word in according storage.
//
// Class that implements this interface is responsible for:
// - returning key word by its index_position in the Index
// - returning collation key for the keyword given its
//   index_position
// - returning position in the storage for a keyword given
//   index_position of this keyword
// - returning Collator object that allows caller
//   to compare keywords of this Index in the right way
//   or to construct CollateKey objects
//
class AbstractIndex
  {
  public:
    virtual ~AbstractIndex() throw(IndexException) {}

    virtual index_type GetNumberOfKeyWords() const throw() = 0;
   
    virtual UnicodeString GetKeyWord(index_type i_index) const throw(IndexException) = 0;

    virtual CollationKey GetKeyWordCollationKey(index_type i_index) const throw(IndexException) = 0;

    virtual AbstractPosition* GetDefinitionPos(index_type i_index) const throw(IndexException) = 0;

    virtual Collator* GetCollator() const throw() = 0;
  };
//------------------------------------------------------------------------------

#endif // ABSTRACT_INDEX_H
