/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef ABSTRACT_STORAGE_H
#define ABSTRACT_STORAGE_H

//------------------------------------------------------------------------------
class AbstractPosition;
//------------------------------------------------------------------------------
// This is an exception class used to report errors
// for all Storages.
class StorageException: public GeneralException
  {
  public:
    StorageException(const std::string& i_err_id,
                     const std::string& i_msg):
      GeneralException("Storage", i_err_id, i_msg) {}
  };
//------------------------------------------------------------------------------
// This is an interface for all Storages - objects that are responsible for
// storing the associated information for keywords: dictionaries in DICT format,
// databases, etc.

// Storage model does not imply the way of actual items storage: they may be
// stored as ``plain'' storages (like one large piece of text), as item-by-item
// storages (like databases), etc.
//
// Class that implements this interface is responsible for:
//   - returning the associated information given its position (position may
//     also include the length of the information if required by storage type,
//     see AbstractPosition class description)
//
class AbstractStorage
  {
  public:
    virtual ~AbstractStorage() throw (StorageException) {}

    virtual std::string GetMimeType() const throw() = 0;

    virtual UnicodeString Get(const AbstractPosition& i_position) const throw (StorageException) = 0;
  };
//------------------------------------------------------------------------------

#endif // ABSTRACT_STORAGE_H
