/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef ABSTRACT_STRATEGY_H
#define ABSTRACT_STRATEGY_H

//------------------------------------------------------------------------------
class AbstractIndex;
class AbstractController;
//------------------------------------------------------------------------------
// This is an exception class used to report errors
// for Strategies.
class StrategyException: public GeneralException
  {
  public:
    StrategyException(const std::string& i_err_id,
                      const std::string& i_msg):
      GeneralException("Strategy", i_err_id, i_msg) {}
  };
//------------------------------------------------------------------------------
// This is an interface for all Strategies - objects that are responsible for
// searching the Index for all keywords that are similar to the given
// keyword.
//
// Class that implements this interface is responsible for:
//   - returning the list of keywords that match given keyword by
//     specific criteria
//
class AbstractStrategy
  {
  public:
    virtual ~AbstractStrategy() throw(StrategyException) {}

    virtual UnicodeString GetStrategyName() const throw() = 0;

    virtual UnicodeString GetStrategyDescription() const throw() = 0;

    virtual void FindKeyWords(std::list<index_type>& o_keywords, AbstractController& io_controller,
                              const AbstractIndex& i_index, const UnicodeString& i_word) const throw(StrategyException) = 0;
  };
//------------------------------------------------------------------------------

#endif // ABSTRACT_STRATEGY_H
