/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef TRANSFORMER_H
#define TRANSFORMER_H

//------------------------------------------------------------------------------
// This is an exception class used to report errors
// for converters.
class TransformerException: public GeneralException
  {
  public:
    TransformerException(const std::string& i_err_id,
                         const std::string& i_msg):
      GeneralException("Transformer", i_err_id, i_msg) {}
  };
//------------------------------------------------------------------------------
// This is an interface for all Transformers - objects that are used
// to convert input between different MIME types (typical examples include
// producing HTML from XML, etc)
//
// Class that implements this interface is responsible for:
//   - returning the modified output produced from the input in
//     the appropriate format
//
class AbstractTransformer
  {
  public:
    virtual ~AbstractTransformer() throw(TransformerException) {}

    virtual UnicodeString GetTransformerName() const throw() = 0;

    virtual std::string GetInputMimeTypeForTransform(const UnicodeString& i_transform) const throw(TransformerException) = 0;

    virtual std::string GetOutputMimeTypeForTransform(const UnicodeString& i_transform) const throw(TransformerException) = 0;

    virtual void AddTransform(const UnicodeString& i_name, const std::string& i_in_mime_type,
                              const std::string& i_out_mime_type, const std::string& i_params) throw(TransformerException) = 0;

    virtual UnicodeString Transform(const UnicodeString& i_input, const UnicodeString& i_transform) const throw(TransformerException) = 0;
  };
//------------------------------------------------------------------------------

#endif // TRANSFORMER_H
