/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "approx_string_matcher.h"

//------------------------------------------------------------------------------
double ApproxStringMatcher::Distance(const UnicodeString& i_str0, const UnicodeString& i_str1, DistanceType i_type) const
  {
  switch (i_type)
    {
    case EDIT_DISTANCE:
      return EditDistance(i_str0, i_str1);

    default:
      assert(0);
      return 0;
    }
  }
//------------------------------------------------------------------------------
double ApproxStringMatcher::EditDistance(const UnicodeString& str1, const UnicodeString& str2) const
  {
  if (str1.length() <= str2.length())
    return EditDistanceImpl(str1, str2);

  return EditDistanceImpl(str2, str1);
  }
//------------------------------------------------------------------------------
// Assumes that str1 is smaller than str2
double ApproxStringMatcher::EditDistanceImpl(const UnicodeString& str1, const UnicodeString& str2) const
  {
  ulong str1_len = (ulong)str1.length(),
        str2_len = (ulong)str2.length();

  if (!str1_len)
    return str2_len;

  assert(str1_len <= str2_len);

  if (m_dist.size() < str1_len)
    m_dist.resize(str1_len);

  for (ulong prev_diag, i = 0; i < str2_len; ++i)
    for (ulong i1 = 0; i1 < str1_len; ++i1)
      {
      ulong repl_cost = (str1[i1] == str2[i])? 0: 1;
      ulong cost_vert = (i1 == 0)? (i + 1) + 1: m_dist[i1 - 1] + 1;
      ulong cost_hor = (i == 0)? (i1 + 1) + 1: m_dist[i1] + 1;
      ulong cost_diag;

      if (i1 == 0)
        cost_diag = i + repl_cost;
      else if (i == 0)
        cost_diag = i1 + repl_cost;
      else
        cost_diag = prev_diag + repl_cost;

      prev_diag = m_dist[i1];
      m_dist[i1] = std::min(cost_vert, std::min(cost_hor, cost_diag));
      }

  return m_dist[str1_len - 1];
  }
//------------------------------------------------------------------------------

