/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef COMMON_H
#define COMMON_H

#ifdef _MSC_VER
// "Clever" M$VC does not understand exception specification :-(
#pragma warning(disable: 4290)
// Forcing value to bool 'true' or 'false' (performance warning)
// Actually, it would not be a problem to deal with that in my code,
// but there are several such warnings in ICU headers :-(
#pragma warning(disable: 4800)
// Avoid deprecation warnings for CRT.
#pragma warning(disable: 4996)
#endif

// General includes:
#include <assert.h>

// STD general includes:
#include <list>
#include <vector>
#include <map>
#include <set>
#include <deque>
#include <queue>
#include <string>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <sstream>
#include <limits>

// BOOST general defines:
#include <boost/lexical_cast.hpp>

// ICU general includes:
#include <unicode/coll.h>
#include <unicode/sortkey.h>
#include <unicode/locid.h>

// Version stuff. Not very clear way,
// but should be enough for package
// of such size.
#define MAJOR_VERSION 1
#define MINOR_VERSION 2
#define PATCH_VERSION 5

#define MAJOR_VERSION_STR "1"
#define MINOR_VERSION_STR "2"
#define PATCH_VERSION_STR "5"

#define VERSION_STR MAJOR_VERSION_STR "." MINOR_VERSION_STR "." PATCH_VERSION_STR

//---------------------------------------------------------------------------
#if defined(_DEBUG) && defined(_MSC_VER)
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>
#endif
//---------------------------------------------------------------------------

#include <xercesc/util/XMLString.hpp>

// These functions should not be used for routinely convertions
// in the main loop of the server! They're just convenient
// tools to simplify config parsing.
inline
std::string AsString(const XMLCh* const ip_str)
  {
  char* p_res_str = xercesc::XMLString::transcode(ip_str);
  std::string res(p_res_str);
  xercesc::XMLString::release(&p_res_str);

  return res;
  }
//---------------------------------------------------------------------------
inline
std::string AsString(const UnicodeString& i_str)
  {
  int32_t len = i_str.extract(0, i_str.length(), (char*)NULL);
  char* p_buf = new char[len + 1];
  i_str.extract(0, i_str.length(), p_buf, len + 1);
  std::string res(p_buf);
  delete[] p_buf;

  return res;
  }
//---------------------------------------------------------------------------
// This function is designed for heavier usage than AsString.
// Be aware that it does not perform real convertion!
inline
std::string AsStringTrunc(const UnicodeString& i_str)
  {
  unsigned long len = i_str.length();
  std::string str;
  str.reserve(len);
  for (unsigned long i = 0; i < len; ++i)
    str.append(1, (char)(i_str[i] & 0xFF));

  return str;
  }
//---------------------------------------------------------------------------

// DictD++ general includes
// (order is important!)
#include "types.h"
#include "logstream.h"
#include "general_exception.h"

#endif // COMMON_H
