/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "convertion.h"
#include "abstract_transformer.h"

//------------------------------------------------------------------------------
Convertion::Convertion(const std::map<UnicodeString, const AbstractTransformer*>& i_transformers):
		      m_transformers(i_transformers)
  {
  }
//------------------------------------------------------------------------------
std::string Convertion::GetInputMimeType() const throw(TransformerException)
  {
  if (m_transform_steps.empty())
    throw TransformerException("NoTransforms", "An attempt to get info for unexisting transform");

  return m_transformers.find(m_transform_steps.front().first) -> second ->
	   GetInputMimeTypeForTransform(m_transform_steps.front().second);
  }
//------------------------------------------------------------------------------
std::string Convertion::GetOutputMimeType() const throw(TransformerException)
  {
  if (m_transform_steps.empty())
    throw TransformerException("NoTransforms", "An attempt to get info for unexisting transform");

  return m_transformers.find(m_transform_steps.back().first) -> second ->
	   GetOutputMimeTypeForTransform(m_transform_steps.back().second);
  }
//------------------------------------------------------------------------------
UnicodeString Convertion::Apply(const UnicodeString& i_input) const throw(TransformerException)
  {
  if (m_transform_steps.empty())
    throw TransformerException("NoTransforms", "An attempt to get info for unexisting transform");

  if (m_transform_steps.size() == 1) // Typical case
    return m_transformers.find(m_transform_steps.front().first) -> second -> Transform(i_input, m_transform_steps.front().second);

  UnicodeString str = i_input;

  for (TransformSteps::const_iterator it = m_transform_steps.begin();
       it != m_transform_steps.end(); ++it)
    str = m_transformers.find(it -> first) -> second -> Transform(str, it -> second);

  return str;
  }
//------------------------------------------------------------------------------
void Convertion::AddTransformStep(const UnicodeString& i_transformer_name, const UnicodeString& i_transform)
  {
  m_transform_steps.push_back(std::pair<UnicodeString, UnicodeString>(i_transformer_name, i_transform));
  }
//------------------------------------------------------------------------------
void Convertion::Validate() const throw(TransformerException)
  {
  if (m_transform_steps.empty())
    throw TransformerException("NoTransforms", "An attempt to get info for unexisting transform");

  // Should get an exception here if this transform does not exist
  std::string type = m_transformers.find(m_transform_steps.front().first) -> second -> GetOutputMimeTypeForTransform(m_transform_steps.front().second);

  if (m_transform_steps.size() == 1)
   return;

  for (TransformSteps::const_iterator it = m_transform_steps.begin(); it != m_transform_steps.end(); ++it)
    {
    std::string type1 = m_transformers.find(it -> first) -> second -> GetInputMimeTypeForTransform(it -> second);

    if (type != type1)
      throw TransformerException("IncompartibleTransforms", "Transforms do not match");

    type = m_transformers.find(it -> first) -> second -> GetOutputMimeTypeForTransform(it -> second);
    }
  }
//------------------------------------------------------------------------------
