/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "dictd_index.h"
#include <unicode/ucnv.h>

//------------------------------------------------------------------------------
void DictdIndex::Initialize() throw(IndexException)
  {
  UErrorCode status = U_ZERO_ERROR;
  mp_collator = Collator::createInstance(m_locale.c_str(), status);

  if (U_FAILURE(status))
    throw IndexException("CollatorCreationFailed", "Failed to create the collator");

  mp_collator -> setStrength(Collator::PRIMARY);

  mp_converter = ucnv_open(m_codepage.c_str(), &status);
  if (U_FAILURE(status))
    throw IndexException("ConverterCreationFailed", "Failed to create the converter");
  }
//------------------------------------------------------------------------------
void DictdIndex::DeInitialize()
  {
  delete mp_index_file;
  delete mp_cache_file;
  delete mp_collator;

  if (mp_converter)
    ucnv_close(mp_converter);
  }
//------------------------------------------------------------------------------
DictdIndex::DictdIndex(const AbstractMemoryMapper* ip_index_file,
		       const AbstractMemoryMapper* ip_cache_file) throw(IndexException):
  mp_index_file(ip_index_file), mp_cache_file(ip_cache_file), mp_collator(NULL),
  mp_converter(NULL)
  {
  const char* p_cache = ip_cache_file -> Get();
  index_type len = ip_cache_file -> Length();

  mp_cache_pos = reinterpret_cast<const index_type*>(p_cache + *reinterpret_cast<const index_type*>(p_cache + len - sizeof(index_type)));
  m_keywords_num = *reinterpret_cast<const index_type*>(p_cache + len - 2*sizeof(index_type));

  assert(p_cache + len - 2*sizeof(index_type) == reinterpret_cast<const char*>(mp_cache_pos + m_keywords_num));

  if (p_cache + len - 2*sizeof(index_type) != reinterpret_cast<const char*>(mp_cache_pos + m_keywords_num))
    throw IndexException("WrongCacheFile", "Cache file is wrong, unable to load");

  m_fl_extended = !strncmp(p_cache, DictdIndexCacher::g_cache_file_ext_sig, strlen(DictdIndexCacher::g_cache_file_ext_sig));

  const char* p_pos = p_cache;
  for (ulong i = 0; i < 3; ++i)
    while (*(p_pos++) != '\n');

  const char* p_old_pos = p_pos;
  while (*(p_pos++) != '\n');
  m_locale = std::string(p_old_pos, p_pos - p_old_pos - 1);
  g_log << DEBUG << "Using locale \"" << m_locale << "\"" << endlog;

  p_old_pos = p_pos;
  while (*(p_pos++) != '\n');
  m_codepage = std::string(p_old_pos, p_pos - p_old_pos - 1);
  g_log << DEBUG << "Using codepage \"" << m_codepage << "\"" << endlog;
  
  Initialize();
  }
//------------------------------------------------------------------------------
DictdIndex::~DictdIndex() throw()
  {
  DeInitialize();
  }
//------------------------------------------------------------------------------
