/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef DICTD_INDEX_H
#define DICTD_INDEX_H

#include "abstract_index.h"
#include "abstract_memory_mapper.h"
#include "dictd_index_cacher.h"
#include "plain_position.h"
#include "index_utils.h"

//------------------------------------------------------------------------------
class AbstractMemoryMapper;
//------------------------------------------------------------------------------
class DictdIndex: public AbstractIndex
  {
  public:
    DictdIndex(const AbstractMemoryMapper* ip_index_file,
	       const AbstractMemoryMapper* ip_cache_file) throw(IndexException);

    virtual ~DictdIndex() throw();

    index_type GetNumberOfKeyWords() const throw();

    UnicodeString GetKeyWord(index_type i_index) const throw(IndexException);

    CollationKey GetKeyWordCollationKey(index_type i_index) const throw();

    AbstractPosition* GetDefinitionPos(index_type i_index) const throw();

    Collator* GetCollator() const throw();

  private:
    const AbstractMemoryMapper* mp_index_file;
    const AbstractMemoryMapper* mp_cache_file;
    std::string m_locale, m_codepage;
    Collator* mp_collator;
    UConverter* mp_converter;
    bool m_fl_extended;
    index_type m_keywords_num;

    // This pointer references to the part of cache that stores index table
    const index_type* mp_cache_pos;
    
  private:
    void Initialize() throw(IndexException);

    void DeInitialize();
  };
//------------------------------------------------------------------------------
// DictdIndex inlines:
//------------------------------------------------------------------------------
inline
Collator* DictdIndex::GetCollator() const throw()
  {
  return mp_collator;
  }
//------------------------------------------------------------------------------
inline
index_type DictdIndex::GetNumberOfKeyWords() const throw()
  {
  return m_keywords_num;
  }
//------------------------------------------------------------------------------
inline
UnicodeString DictdIndex::GetKeyWord(index_type i_index) const throw(IndexException)
  {
  assert(i_index < GetNumberOfKeyWords());

  if (m_fl_extended)
    {
    DictdCacheInfoExtended info(mp_cache_file -> Get() + mp_cache_pos[i_index]);
    return info.GetKeyWord();
    }
  else
    {
    DictdCacheInfoBasic info(mp_cache_file -> Get() + mp_cache_pos[i_index]);
    UErrorCode status = U_ZERO_ERROR;
    UnicodeString str(mp_index_file -> Get() + info.GetKeyWordPos(), (int32_t)info.GetKeyWordLength(), mp_converter, status);

    if (U_FAILURE(status))
      throw IndexException("ConverionFailed", "Failed to convert keyword");

    return str;
    }
  }
//------------------------------------------------------------------------------
inline
CollationKey DictdIndex::GetKeyWordCollationKey(index_type i_index) const throw()
  {
  assert(i_index < GetNumberOfKeyWords());

  if (m_fl_extended)
    {
    DictdCacheInfoExtended info(mp_cache_file -> Get() + mp_cache_pos[i_index]);
    return info.GetCollationKey();
    }
  else
    {
    DictdCacheInfoBasic info(mp_cache_file -> Get() + mp_cache_pos[i_index]);
    return info.GetCollationKey();
    }
  }
//------------------------------------------------------------------------------
inline
AbstractPosition* DictdIndex::GetDefinitionPos(index_type i_index) const throw()
  {
  assert(i_index < GetNumberOfKeyWords());

  if (m_fl_extended)
    {
    DictdCacheInfoExtended info(mp_cache_file -> Get() + mp_cache_pos[i_index]);
    return new PlainPosition(info.GetDefinitionPos(), info.GetDefinitionLength());
    }
  else
    {
    DictdCacheInfoBasic info(mp_cache_file -> Get() + mp_cache_pos[i_index]);
    return new PlainPosition(info.GetDefinitionPos(), info.GetDefinitionLength());
    }
  }
//------------------------------------------------------------------------------

#endif // DICTD_INDEX_H
