/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef DICTD_INDEX_CACHER_H
#define DICTD_INDEX_CACHER_H

//------------------------------------------------------------------------------
class IndexException;
class AbstractMemoryMapper;
//------------------------------------------------------------------------------
class DictdIndexCacher
  {
  public:
    static const char* g_cache_file_basic_sig;
    static const char* g_cache_file_ext_sig;

  public:
    DictdIndexCacher(const std::string& i_locale, const std::string& i_codepage,
		     const AbstractMemoryMapper& i_index_file,
		     const std::string& i_index_id, bool i_fl_extended);
		     
    ~DictdIndexCacher() throw();

    bool IsCacheValid(std::iostream& io_cache) const;

    void CreateCache(std::iostream& io_cache) const;

  private:
    const AbstractMemoryMapper& m_index_file;
    bool m_fl_extended;
    std::string m_locale, m_codepage, m_index_id;
    Collator* mp_collator;
    UConverter* mp_converter;

  private:
    void Initialize() throw(IndexException); 

    void DeInitialize();

    void GetIndexEntry(storage_type& o_start, storage_type& o_length,
		       index_type& o_keyword_length, UnicodeString& o_keyword,
		       const char*& p_cur_pos) const throw(IndexException);
  };
//------------------------------------------------------------------------------
struct DictdCacheInfoBasic
  {
  const char* mp_pos;

  DictdCacheInfoBasic(const char* ip_pos): mp_pos(ip_pos) {}

  static void Write(std::ostream& os,
		    index_type i_keyword_pos, int32_t i_keyword_len,
		    storage_type i_def_start, storage_type i_def_len,
		    const CollationKey& i_key);

  index_type GetKeyWordPos() const;

  index_type GetKeyWordLength() const;

  storage_type GetDefinitionPos() const;

  storage_type GetDefinitionLength() const;

  CollationKey GetCollationKey() const;
  };
//------------------------------------------------------------------------------
struct DictdCacheInfoExtended
  {
  const char* mp_pos;

  DictdCacheInfoExtended(const char* ip_pos): mp_pos(ip_pos) {}

  static void Write(std::ostream& os,
		    storage_type i_def_start, storage_type i_def_len,
		    const CollationKey& i_key, const UnicodeString& i_keyword);

  storage_type GetDefinitionPos() const;

  storage_type GetDefinitionLength() const;

  CollationKey GetCollationKey() const;

  UnicodeString GetKeyWord() const;
  };
//------------------------------------------------------------------------------
// DictdCacheInfoBasic inlines:
//------------------------------------------------------------------------------
inline
index_type DictdCacheInfoBasic::GetKeyWordPos() const
  {
  return *reinterpret_cast<const index_type*>(mp_pos);
  }
//------------------------------------------------------------------------------
inline
index_type DictdCacheInfoBasic::GetKeyWordLength() const
  {
  return *reinterpret_cast<const index_type*>(mp_pos + sizeof(index_type));
  }
//------------------------------------------------------------------------------
inline
storage_type DictdCacheInfoBasic::GetDefinitionPos() const
  {
  return *reinterpret_cast<const storage_type*>(mp_pos + 2*sizeof(index_type));
  }
//------------------------------------------------------------------------------
inline
storage_type DictdCacheInfoBasic::GetDefinitionLength() const
  {
  return *reinterpret_cast<const storage_type*>(mp_pos + 2*sizeof(index_type) + sizeof(storage_type));
  }
//------------------------------------------------------------------------------
inline
CollationKey DictdCacheInfoBasic::GetCollationKey() const
  {
  int32_t key_size = *reinterpret_cast<const int32_t*>(mp_pos + 2*sizeof(index_type) + 2*sizeof(storage_type));

  return CollationKey(reinterpret_cast<const uint8_t*>(mp_pos + 2*sizeof(index_type) + 2*sizeof(storage_type)
						       + sizeof(int32_t)), key_size);
  }
//------------------------------------------------------------------------------
// DictdCacheInfoExtended inlines:
//------------------------------------------------------------------------------
inline
storage_type DictdCacheInfoExtended::GetDefinitionPos() const
  {
  return *reinterpret_cast<const storage_type*>(mp_pos);
  }
//------------------------------------------------------------------------------
inline
storage_type DictdCacheInfoExtended::GetDefinitionLength() const
  {
  return *reinterpret_cast<const storage_type*>(mp_pos + sizeof(storage_type));
  }
//------------------------------------------------------------------------------
inline
CollationKey DictdCacheInfoExtended::GetCollationKey() const
  {
  int32_t key_size = *reinterpret_cast<const int32_t*>(mp_pos + 2*sizeof(storage_type));

  return CollationKey(reinterpret_cast<const uint8_t*>(mp_pos + 2*sizeof(storage_type)
						       + 2*sizeof(int32_t)), key_size);
  }
//------------------------------------------------------------------------------
inline
UnicodeString DictdCacheInfoExtended::GetKeyWord() const
  {
  int32_t key_size = *reinterpret_cast<const int32_t*>(mp_pos + 2*sizeof(storage_type));
  int32_t keyword_size = *reinterpret_cast<const int32_t*>(mp_pos + 2*sizeof(storage_type)
							   + sizeof(int32_t));

  return UnicodeString(FALSE, reinterpret_cast<const UChar*>(mp_pos + 2*sizeof(storage_type)
							     + 2*sizeof(int32_t)
							     + key_size*sizeof(uint8_t)), keyword_size);
  }
//------------------------------------------------------------------------------
#endif // DICTD_INDEX_CACHER_H
