/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "dictd_utils.h"
#include "crypto/dictd_md5.h"

//------------------------------------------------------------------------------
bool Tools::DequoteArgs(std::vector<std::string>& o_list, const std::string& i_str)
  {
  char quote_type = 0;
  bool need_space = false, fl_first = true;
  std::string::size_type len = i_str.length();
  
  o_list.clear();
  
  if (!len)
    return true;
    
  o_list.push_back("");
  
  for (std::string::size_type i = 0; i < len; ++i)
    {
    if (!quote_type && (i_str[i] == ' ' || i_str[i] == '\t'))
      {
      if (!fl_first)
        o_list.push_back("");
      fl_first = true;
      need_space = false;
      continue;
      }
    
    if (need_space)
      return false;
    
    if (i_str[i] == '\\' && i < len)
      {
      o_list.back() += i_str[++i];
      fl_first = false;
      continue;
      }
    
    if (i_str[i] == '\'' || i_str[i] == '\"')
      {
      if (fl_first && !quote_type)
        {
        quote_type = i_str[i];
	      fl_first = false;
	      continue;
        }
      else if (quote_type == i_str[i])
        {
        quote_type = 0;
        need_space = true;
	      fl_first = false;
	      continue;
        }
      else if (!quote_type)
        return false;
      }
    
    o_list.back() += i_str[i];
    fl_first = false;
    }
  
  return true;
  }
//------------------------------------------------------------------------------
UnicodeString Tools::QuoteString(const UnicodeString& i_str)
  {
  bool fl_quote = false;

  for (int32_t i = 0; i < i_str.length(); ++i)
    if (i_str[i] <= 0x20 || i_str[i] == '\"' || i_str[i] == '\'')
      {
      fl_quote = true;
      break;
      }

  if (!fl_quote)
    return i_str;

  UnicodeString res;
  res += "\"";

  for (int32_t i = 0; i < i_str.length(); ++i)
    if (i_str[i] == '\"')
      res += "\\\"";
    else
      res += i_str[i];

  res += "\"";

  return res;
  }
//------------------------------------------------------------------------------
std::string Tools::GetMD5(const std::string& i_str)
  {
  md5_context ctx;
  unsigned char md5sum[16];

  md5_starts(&ctx);
  md5_update(&ctx, (uint8_t*)i_str.c_str(), (uint32_t)i_str.size());
  md5_finish(&ctx, md5sum);

  std::stringstream res;

  res << std::hex << std::nouppercase;

  for (ulong j = 0; j < 16; ++j)
    {
    if (md5sum[j] < 16) res << '0';
    res << (int)md5sum[j];
    }

  return res.str();
  }
//------------------------------------------------------------------------------
