/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef DICTIONARY_H
#define DICTIONARY_H

#include "mime_type.h"
//------------------------------------------------------------------------------
class AbstractIndex;
class AbstractStorage;
class AbstractStrategy;
class AbstractController;
struct AccessRights;
class Convertion;
//------------------------------------------------------------------------------
// This is an exception class used to report errors
// for dictionaries.
class DictionaryException: public GeneralException
  {
  public:
    DictionaryException(const std::string& i_err_id,
			    const std::string& i_msg):
      GeneralException("Dictionary", i_err_id, i_msg) {}
  };
//------------------------------------------------------------------------------
// This is an Dictionary object, that is responsible for the complete information
// about particular dictionary. It contains appropriate Index and Storage objects
// as well as list of allowed convertions.
//
class Dictionary
  {
  public:
    typedef std::pair<UnicodeString, UnicodeString> DefinitionItem;
    typedef std::list<DefinitionItem> Definitions;

    Dictionary(const UnicodeString& i_name, const UnicodeString& i_descr, const std::string& i_def_out_mime_type,
               const AbstractIndex* ip_index, const AbstractStorage* ip_storage,
               const std::list<Convertion*>* mp_convertions,
               const AccessRights* mp_access_rights) throw(DictionaryException);

    ~Dictionary() throw();

    void InitConvertions() throw(DictionaryException);

    UnicodeString GetDictionaryName() const throw();

    UnicodeString GetDictionaryDescription() const throw();

    UnicodeString GetDictionaryStatistics() const throw();

    std::string GetMimeType() const throw();

    const AccessRights& GetAccessRights() const throw();

    bool IsStrategySupported(const AbstractStrategy& i_strategy) const throw();

    void FindKeyWords(std::list<UnicodeString>& o_keywords, AbstractController& io_controller, const UnicodeString& i_word, const AbstractStrategy& i_strategy) const throw(DictionaryException);

    void FindDefinitions(Definitions& o_definitions, AbstractController& io_controller,
                         const UnicodeString& i_word, const AbstractStrategy& i_strategy,
                         const std::list<MimeType>& i_mime_types, bool i_fl_show_mime_header) const throw(DictionaryException);

    void Validate() const throw(DictionaryException);

    std::string ProduceOutputMimeType(const std::list<MimeType>& i_mime_types) const;

  private:
    UnicodeString m_name, m_descr;
    std::string m_def_out_mime_type;
    const AbstractIndex* mp_index;
    const AbstractStorage* mp_storage;
    const std::list<Convertion*>* mp_convertions;
    std::map<std::string, Convertion*> m_convertions;
    const AccessRights* mp_access_rights;
  };
//------------------------------------------------------------------------------

#endif // DICTIONARY_H
