/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "dzip_storage.h"
#include "zlib_constants.h"
#include "plain_position.h"
#include "abstract_memory_mapper.h"
#include <unicode/ucnv.h>

//------------------------------------------------------------------------------
//
// TODO: all code of DZip Storage is not very clean, it's a copy/paste from several
// years old prototype. Refactor !!! $AT
//
//------------------------------------------------------------------------------
DZipStorage::DZipStorage(const std::string& i_codepage, const std::string& i_mime_type, const AbstractMemoryMapper* ip_storage,
			 ulong i_cache_size) throw(StorageException):
  m_codepage(i_codepage), m_mime_type(i_mime_type), mp_storage(ip_storage), m_cache_size(i_cache_size)
  {
  UErrorCode status = U_ZERO_ERROR;
  mp_converter = ucnv_open(m_codepage.c_str(), &status);

  if (U_FAILURE(status))
    throw StorageException("ConverterCreationFailed", "Failed to create the converter");

  storage_type shift = 0;
  
  const unsigned char* pu_dict = reinterpret_cast<const unsigned char*>(mp_storage -> Get());
  
  if (pu_dict[shift++] != GZ_MAGIC1 || pu_dict[shift++] != GZ_MAGIC2)
    throw StorageException("DZipWrongFormat", "Wrong DZip file format: magics do not match");
  
  m_header.m_method = pu_dict[shift++];
  
  m_header.m_flags = pu_dict[shift++];
  
  m_header.m_mtime = pu_dict[shift++];
  m_header.m_mtime |= pu_dict[shift++] << 8;
  m_header.m_mtime |= pu_dict[shift++] << 16;
  m_header.m_mtime |= pu_dict[shift++] << 24;
  
  m_header.m_extra_flags = pu_dict[shift++];
  
  m_header.m_os = pu_dict[shift++];

  m_header.m_header_length = GZ_XLEN - 1;
  
  if (! (m_header.m_flags & GZ_FEXTRA))
    throw StorageException("DZipWrongFormat", "Wrong DZip file format: extra header section not found");
  else
    {
    unsigned extra_length = pu_dict[shift++];
    extra_length |= pu_dict[shift++] << 8;
    
    m_header.m_header_length += extra_length + 2;
    
    char si1 = pu_dict[shift++];
    char si2 = pu_dict[shift++];
			  
    if (si1 != GZ_RND_S1 || si2 != GZ_RND_S2)
      throw StorageException("DZipWrongFormat", "Wrong DZip file format: RND flag does not match");
    else
      {
      unsigned sub_length = pu_dict[shift++];
      sub_length |= pu_dict[shift++] << 8;
      
      unsigned version = pu_dict[shift++];
      version |= pu_dict[shift++] << 8;
      
      if (version != 1)
	throw StorageException("DZipWrongFormat", "Wrong DZip file format: wrong DZip extra info version");

      m_chunk_size = pu_dict[shift++];
      m_chunk_size |= pu_dict[shift++] << 8;
      
      unsigned chunks_count = pu_dict[shift++];
      chunks_count |= pu_dict[shift++] << 8;

      g_log << TRACE << "Chunk size = " << m_chunk_size << endlog;
      g_log << TRACE << "Chunks number = " << chunks_count << endlog;
      
      m_chunk_index.reserve(chunks_count);
    
      ChunkEntry chunk_entry;
      m_chunk_index.reserve(chunks_count);

      for (unsigned i = 0; i < chunks_count; ++i)
	{
	chunk_entry.m_compressed_size = pu_dict[shift++];
	chunk_entry.m_compressed_size |= pu_dict[shift++] << 8;
      
	m_chunk_index.push_back(chunk_entry);
	}
    } // MAGIC is OK
  
    if (m_header.m_flags & GZ_FNAME)
      {
      for (int c; (c = pu_dict[shift++]) && shift < mp_storage -> Length(); m_header.m_orig_file_name += c);
      m_header.m_header_length += (ulong)(m_header.m_orig_file_name.length() + 1);
      }
   
    if (m_header.m_flags & GZ_COMMENT)
      {
      for (int c; (c = pu_dict[shift++]) && shift < mp_storage -> Length(); m_header.m_comment += c);
      m_header.m_header_length += (ulong)(m_header.m_comment.length() + 1);
      }

    if (m_header.m_flags & GZ_FHCRC)
      {
      shift += 2;
      m_header.m_header_length += 2;
      }

    if (shift != m_header.m_header_length + 1)
      throw StorageException("DZipWrongFormat", "Wrong DZip file format: header length mismatch");

    shift = mp_storage -> Length() - 8;
   
    m_header.m_crc = pu_dict[shift++];
    m_header.m_crc |= pu_dict[shift++] <<  8;
    m_header.m_crc |= pu_dict[shift++] << 16;
    m_header.m_crc |= pu_dict[shift++] << 24;
   
    m_header.m_length = pu_dict[shift++];
    m_header.m_length |= pu_dict[shift++] <<  8;
    m_header.m_length |= pu_dict[shift++] << 16;
    m_header.m_length |= pu_dict[shift++] << 24;
   
    m_header.m_compressed_length = shift;

    for (ulong offset = m_header.m_header_length + 1, i = 0; i < m_chunk_index.size(); ++i)
       {
       m_chunk_index[i].m_position = offset;
       offset += m_chunk_index[i].m_compressed_size;
       }
  } // extra flags are present

  mp_out_buffer = new Bytef[m_chunk_size];
  
  m_stamp = 0;
  
  m_zstream.zalloc = NULL;
  m_zstream.zfree = NULL;
  m_zstream.opaque = NULL;
  m_zstream.next_in = NULL;
  m_zstream.avail_in = 0;
  m_zstream.next_out = NULL;
  m_zstream.avail_out = 0;

  if (inflateInit2(&m_zstream, -15) != Z_OK)  // "-15" to suppress zlib header
    throw StorageException("ZLibInitFailed", "ZLib init failed");
  }
//------------------------------------------------------------------------------
DZipStorage::~DZipStorage() throw(StorageException)
  {
  delete mp_storage;
  ucnv_close(mp_converter);

  delete[] mp_out_buffer;
  
  if (inflateEnd(&m_zstream) != Z_OK)
    throw StorageException("ZLibDeInitFailed", "Failed to deinitialize ZLib");
  }
//------------------------------------------------------------------------------
UnicodeString DZipStorage::Get(const AbstractPosition& i_position) const throw(StorageException)
  {
  const PlainPosition* p_pos = dynamic_cast<const PlainPosition*>(&i_position);
  if (!p_pos)
    throw StorageException("WrongPositionType", "Plain storage expects position of PlainPosition type");

  ulong chunk_beg_num = p_pos -> m_pos / m_chunk_size,
	chunk_end_num = (p_pos -> m_pos + p_pos -> m_length - 1) / m_chunk_size;
  
  const std::string& str = GetChunk(chunk_beg_num);
  
  storage_type shift = p_pos -> m_pos - chunk_beg_num * m_chunk_size,
	       beg_size = (storage_type)str.size() - shift;

  std::string res_str = str.substr(shift, std::min(beg_size, p_pos -> m_length));
  
  for (ulong chunk_num = chunk_beg_num + 1; chunk_num < chunk_end_num; ++chunk_num)
    res_str += GetChunk(chunk_num);
  
  if (chunk_beg_num != chunk_end_num)
    {
    const std::string& str = GetChunk(chunk_end_num);
  
    res_str += str.substr(0, p_pos -> m_pos + p_pos -> m_length - chunk_end_num * m_chunk_size);
    }

  UErrorCode status = U_ZERO_ERROR;
  UnicodeString out_str(res_str.c_str(), (int32_t)res_str.size(), mp_converter, status);

  if (U_FAILURE(status))
    throw StorageException("ConverionFailed", "Failed to create new string with convertion");

  return out_str;
  }
//------------------------------------------------------------------------------
const std::string& DZipStorage::GetChunk(storage_type i_index) const throw(StorageException)
  {
  boost::mutex::scoped_lock lock(m_mutex);
  assert(i_index < m_chunk_index.size());

  ChunkCacheEntry entry(i_index);
  ChunkCache::iterator iter = std::find(m_cache.begin(), m_cache.end(), entry);
  
  if (iter != m_cache.end())
    {
    iter -> m_stamp = m_stamp++;
    //g_log << DEBUG << "DZip cache hit" << endlog;
    return iter -> m_item;
    }

  //g_log << DEBUG << "DZip cache miss" << endlog;
  
  ChunkCacheEntry* p_cache_item;
  
  if (m_cache.size() == m_cache_size)  // test if we should clear oldest item in cache
    p_cache_item = &*std::min(m_cache.begin(), m_cache.end());
  else
    {
    m_cache.push_back(ChunkCacheEntry());
    p_cache_item = &m_cache.back();
    }
    
  p_cache_item -> m_stamp = m_stamp++;
  p_cache_item -> m_number = i_index;

  m_zstream.next_in = const_cast<Bytef*>(reinterpret_cast<const Bytef*>(mp_storage -> Get() + m_chunk_index[i_index].m_position));
  m_zstream.avail_in = m_chunk_index[i_index].m_compressed_size;
  m_zstream.next_out = mp_out_buffer;
  m_zstream.avail_out = m_chunk_size;

  if (inflate(&m_zstream, Z_PARTIAL_FLUSH) != Z_OK || m_zstream.avail_in != 0)
    throw StorageException("ZLibFailed", "Failed to extract the chunk");
  
  p_cache_item -> m_item.assign(reinterpret_cast<char*>(mp_out_buffer), m_chunk_size - m_zstream.avail_out);
  
  return p_cache_item -> m_item;
  }
//------------------------------------------------------------------------------        
