/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef DZIP_STORAGE_H
#define DZIP_STORAGE_H

#include "abstract_storage.h"
#include <zlib.h>

#include <boost/thread/mutex.hpp>

//------------------------------------------------------------------------------
class AbstractMemoryMapper;
class AbstractPosition;
//------------------------------------------------------------------------------
class DZipStorage: public AbstractStorage
  {
  public:
    DZipStorage(const std::string& i_codepage, const std::string& i_mime_type,
		const AbstractMemoryMapper* ip_storage,
		const ulong i_cache_size = 32) throw(StorageException);

    virtual ~DZipStorage() throw(StorageException);

    virtual std::string GetMimeType() const throw();

    virtual UnicodeString Get(const AbstractPosition& i_position) const throw(StorageException);

  private:
    struct  ChunkEntry
      {
      storage_type m_position, m_compressed_size;
      };
    
    struct ChunkCacheEntry
      {
      storage_type m_number, m_stamp;
      std::string m_item;
      
      ChunkCacheEntry() {}
      
      ChunkCacheEntry(int i_number): m_number(i_number) {}
      
      // cache support
      bool operator < (const ChunkCacheEntry& i_entry)
	{
	return m_stamp < i_entry.m_stamp;
	}
      
      // searching of item in cache support
      bool operator == (const ChunkCacheEntry& i_entry)
	{
	return m_number == i_entry.m_number;
	}  
    };
    
    typedef std::vector<ChunkCacheEntry> ChunkCache;

    struct DZipHeader
      {
      char m_method, m_flags, m_extra_flags, m_os;
      ulong m_mtime, m_header_length, m_compressed_length, m_length, m_crc;
      std::string m_orig_file_name, m_comment;
      };

    std::string m_codepage, m_mime_type;
    const AbstractMemoryMapper* mp_storage;
    UConverter* mp_converter;

    DZipHeader m_header;
    
    storage_type m_chunk_size, m_cache_size;

    std::vector<ChunkEntry> m_chunk_index;

    // Cache-specific stuff, shoud be mutable to allow
    // const access to class
    mutable storage_type m_stamp;
    mutable z_stream m_zstream;
    mutable ChunkCache m_cache;
    mutable Bytef* mp_out_buffer;

    mutable boost::mutex m_mutex;
    
    const std::string& GetChunk(ulong i_index) const throw(StorageException);
  };
//------------------------------------------------------------------------------
inline
std::string DZipStorage::GetMimeType() const throw()
  {
  return m_mime_type;
  }
//------------------------------------------------------------------------------

#endif // DZIP_STORAGE_H
