/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef GENERAL_EXEPTION_H
#define GENERAL_EXEPTION_H

#include <errno.h>

//------------------------------------------------------------------------------
// This is a base class for all exeptions used in DictD++.

class GeneralException
  {
  friend std::ostream& operator << (std::ostream& os, const GeneralException& ex);

  public:
    const std::string& GetID() const;

  protected:
    GeneralException(const std::string& i_module,
		     const std::string& i_err_id,
		     const std::string& i_msg): m_module(i_module),
						m_err_id(i_err_id), m_msg(i_msg)
      {
      AddReportFromOS();
      g_log << ERR << *this << endlog;
      }

  private:
    // m_module contains name of the module; should be set up in
    // derivative class constructor;
    // m_err_ind contains short, unique (within module) text ID of an error, like "FileNotFound";
    // m_msg contains user-readable description of an error;
    // m_sys_msg and m_sys_err_code are filled automatically
    std::string m_module, m_err_id, m_msg, m_sys_msg;
    int m_sys_err_code;

  private:
    void AddReportFromOS();
  };
//-----------------------------------------------------------------------------
inline
const std::string& GeneralException::GetID() const
  {
  return m_err_id;
  }
//-----------------------------------------------------------------------------
inline
void GeneralException::AddReportFromOS()
  {
  m_sys_err_code = errno;

  if (m_sys_err_code == 0)
    return;

#ifdef _WIN32
  LPVOID lpMsgBuf;

  FormatMessage( 
    FORMAT_MESSAGE_ALLOCATE_BUFFER | 
    FORMAT_MESSAGE_FROM_SYSTEM | 
    FORMAT_MESSAGE_IGNORE_INSERTS,
    NULL,
    m_sys_err_code,
    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
    (LPTSTR) &lpMsgBuf,
    0,
    NULL 
  );

  m_sys_msg.assign((LPCTSTR)lpMsgBuf);

  // Remove new line (\r\n):
  m_sys_msg.erase(m_sys_msg.end() - 2, m_sys_msg.end());

  LocalFree( lpMsgBuf );
#else
  m_sys_msg.assign(strerror(m_sys_err_code));
#endif
  }
//-----------------------------------------------------------------------------
inline std::ostream& operator << (std::ostream& os, const GeneralException& ex)
  {
  os << "Exception in module " << ex.m_module << std::endl;
  os << "[" << ex.m_err_id << "]: " << ex.m_msg << std::endl;

  if (!ex.m_sys_msg.empty())
    os << "OS error code: 0x" << std::hex << ex.m_sys_err_code << std::dec
       << ", OS error description: " << ex.m_sys_msg << std::endl;

  return os;
  }
//--------------------------------------------------------------------

#endif // GENERAL_EXEPTION_H

