/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef INDEX_UTILS_H
#define INDEX_UTILS_H

#include "abstract_index.h"
#include <boost/iterator/iterator_facade.hpp>

//------------------------------------------------------------------------------
class IndexIterator:
	public boost::iterator_facade<IndexIterator,
                                      index_type,
                                      std::random_access_iterator_tag,
                                      index_type,
                                      signed_index_type>
  {
  public:
    IndexIterator(): m_pos(0) {}

    IndexIterator(index_type index): m_pos(index) {}
    
  private:
    friend class boost::iterator_core_access;

    void increment() { ++m_pos; }

    void decrement() { --m_pos; }

    void advance(difference_type n) { m_pos += n; }

    difference_type distance_to(IndexIterator it) const { return it.m_pos - m_pos; }

    index_type dereference() const { return m_pos; }

    bool equal(IndexIterator it) const { return m_pos == it.m_pos; }

  private:
    index_type m_pos;
  };
//------------------------------------------------------------------------------
inline
bool operator < (const CollationKey& i_str0, const CollationKey& i_str1)
  {
  return i_str0.compareTo(i_str1) == Collator::LESS;
  }
//------------------------------------------------------------------------------
class IndexItemsComparator
  {
  public:
    IndexItemsComparator(const AbstractIndex* ip_index) { mp_index = ip_index; }

    bool operator() (const index_type& i_ind0, const CollationKey& i_key) const
      {
      return mp_index -> GetKeyWordCollationKey(i_ind0) < i_key;
      }

    bool operator() (const CollationKey& i_key, const index_type& i_ind1) const
      {
      return i_key < mp_index -> GetKeyWordCollationKey(i_ind1);
      }

    // STL debug code in VC8 requires this overload
    bool operator() (const index_type& i_ind0, const index_type& i_ind1) const
      {
      return mp_index -> GetKeyWordCollationKey(i_ind0) < mp_index -> GetKeyWordCollationKey(i_ind1);
      }

  private:
    const AbstractIndex* mp_index;
  };
//------------------------------------------------------------------------------

#endif // INDEX_UTILS_H

