/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "levinshtein_match_strategy.h"
#include "abstract_index.h"
#include "approx_string_matcher.h"
#include "abstract_controller.h"

//------------------------------------------------------------------------------
LevinshteinMatchStrategy::LevinshteinMatchStrategy(ulong i_max_dist): m_max_dist(i_max_dist)
 {
 }
//------------------------------------------------------------------------------
LevinshteinMatchStrategy::~LevinshteinMatchStrategy() throw()
 {
 }
//------------------------------------------------------------------------------
UnicodeString LevinshteinMatchStrategy::GetStrategyName() const throw()
  {
  return (std::string("lev-") + boost::lexical_cast<std::string>(m_max_dist)).c_str();
  }
//------------------------------------------------------------------------------
UnicodeString LevinshteinMatchStrategy::GetStrategyDescription() const throw()
  {
  return (std::string("Match words up to Levinshtein distance ") + boost::lexical_cast<std::string>(m_max_dist)).c_str();
  }
//------------------------------------------------------------------------------
void LevinshteinMatchStrategy::FindKeyWords(std::list<index_type>& o_keywords, AbstractController& io_controller,
                                            const AbstractIndex& i_index, const UnicodeString& i_word) const throw(StrategyException)
  {
  index_type index_size = i_index.GetNumberOfKeyWords();
  ApproxStringMatcher matcher;

  for (index_type i = 0; i < index_size; ++i)
    {
    if (!io_controller.IsMatchAllowed())
      break;

    UnicodeString keyword = i_index.GetKeyWord(i);

    if ((ulong)std::abs(keyword.length() - i_word.length()) <= m_max_dist &&
	matcher.Distance(i_word, keyword) <= m_max_dist)
      {
      o_keywords.push_back(i);
      io_controller.OneMoreMatch();
      }
    }
  }
//------------------------------------------------------------------------------
