/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "memory_mapped_file.h"
#include <sys/stat.h>

#ifndef _WIN32
#include <sys/types.h>
#include <sys/mman.h>
#include <fcntl.h>
#endif

//------------------------------------------------------------------------------
MemoryMappedFile::MemoryMappedFile(const std::string& i_file_name) throw(IOException): m_file_name(i_file_name)
  {
#ifdef _WIN32
  mh_file = CreateFile(i_file_name.c_str(), GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);

  if (!mh_file)
    throw IOException("CannotOpenFile", "Cannot open file " + i_file_name);

  mh_map_file = CreateFileMapping(mh_file, NULL, PAGE_READONLY, 0, 0, "");
 
  if (!mh_map_file) 
    throw IOException("CannotCreateMappingForFile", "Cannot create memory mapping for the file " + i_file_name);

  mp_buffer = (char*)MapViewOfFile(mh_map_file, FILE_MAP_READ, 0, 0, 0);
 
  if (!mp_buffer)
    throw IOException("CannotMmapFile", "Cannot mmap file " + i_file_name);

  struct stat stat_info;
  if (stat(i_file_name.c_str(), &stat_info))
    throw IOException("CannotStatFile", "Cannot stat file " + i_file_name);
  m_length = stat_info.st_size;
#else
  struct stat stat_info;
  if (stat(i_file_name.c_str(), &stat_info))
    throw IOException("CannotStatFile", "Cannot stat file " + i_file_name);
    
  m_length = stat_info.st_size;

  mh_file = open(i_file_name.c_str(), O_RDONLY);
  if (mh_file == -1)
    throw IOException("CannotOpenFile", "Cannot open file " + i_file_name);
    
  int page_size = getpagesize();
  g_log << DEBUG << "Requesting mapping of " << m_length << " bytes, pagesize is " << page_size << endlog;
  mp_buffer = (char*)mmap(NULL, (m_length / page_size + 1)*page_size, PROT_READ, MAP_SHARED, mh_file, 0);
  if (mp_buffer == MAP_FAILED)
    throw IOException("CannotMMapFile",  "Cannot mmap file " + i_file_name);
#endif
  }
//------------------------------------------------------------------------------
MemoryMappedFile::~MemoryMappedFile() throw(IOException)
  {
#ifdef _WIN32
  if (!UnmapViewOfFile(mp_buffer))
    throw IOException("CannotUnmapFile", "Cannot unmap file" + m_file_name);

  CloseHandle(mh_map_file);
  CloseHandle(mh_file);
#else
  if (munmap(mp_buffer, m_length))
    throw IOException("CannotUnmapFile", "Cannot unmap file" + m_file_name);
    
  close(mh_file);
#endif
  }
//------------------------------------------------------------------------------
