/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef MIME_TYPE_H
#define MIME_TYPE_H

//------------------------------------------------------------------------------
// This class is used to hold information about requested
// by the client MIME type
struct MimeType
  {
  std::string m_type, m_sub_type;
  double m_preferred;

  bool operator < (const MimeType& i_mime_type) const;

  MimeType() { Construct("*/*"); }

  MimeType(const std::string& i_str) { Construct(i_str); }

  bool Construct(const std::string& i_mime_type);

  bool Matches(const std::string& i_mime_type) const;

  std::string AsString(bool i_fl_show_pref = false) const;

  private:
    void SkipSpaces(std::string::size_type& i, const std::string& i_str) const;
  };
//------------------------------------------------------------------------------
inline
bool MimeType::operator < (const MimeType& i_mime_type) const
  {
  if (m_preferred != i_mime_type.m_preferred)
    return m_preferred > i_mime_type.m_preferred;

  if (m_type != i_mime_type.m_type)
    {
    if (m_type == "*")
      return false;
    return m_type.compare(i_mime_type.m_type) > 0;
    }

  if (m_sub_type != i_mime_type.m_sub_type && m_type == "*")
    return false;

  return m_sub_type.compare(i_mime_type.m_sub_type) > 0;
  }
//------------------------------------------------------------------------------
inline
void MimeType::SkipSpaces(std::string::size_type& i, const std::string& i_str) const
  {
  for (; i < i_str.size() && isspace(i_str[i]); ++i);
  }
//------------------------------------------------------------------------------
// Mmm, code looks not very nice :-(
// Subset of HTTP:Accept header functionality is implemented
// (no additional tokens are supported)
//
inline
bool MimeType::Construct(const std::string& i_mime_type)
  {
  std::string::size_type i = 0;

  SkipSpaces(i, i_mime_type);
  if (i == i_mime_type.size())
    return false;

  m_type.clear();

  for (; i < i_mime_type.size() && !isspace(i_mime_type[i]) && i_mime_type[i] != '/'; ++i)
    m_type += i_mime_type[i];

  SkipSpaces(i, i_mime_type);
  if (i == i_mime_type.size())
    return false;
    
    
  if (i_mime_type[i] != '/')
    return false;

  ++i;

  SkipSpaces(i, i_mime_type);
  if (i == i_mime_type.size())
    return false;

  m_sub_type.clear();

  for (; i < i_mime_type.size() && !isspace(i_mime_type[i]) && i_mime_type[i] != ';'; ++i)
    m_sub_type += i_mime_type[i];

  SkipSpaces(i, i_mime_type);
  if (i == i_mime_type.size())
    {
    m_preferred = 1.0;
    return true;
    }

  if (i_mime_type[i] != ';')
    return false;

  ++i;

  SkipSpaces(i, i_mime_type);
  if (i == i_mime_type.size())
    return false;

  if (i_mime_type[i] != 'q')
    return false;

  ++i;

  SkipSpaces(i, i_mime_type);
  if (i == i_mime_type.size())
    return false;

  if (i_mime_type[i] != '=')
    return false;

  ++i;

  SkipSpaces(i, i_mime_type);
  if (i == i_mime_type.size())
    return false;

  std::string val;
  
  for (; i < i_mime_type.size() && !isspace(i_mime_type[i]); ++i)
    val += i_mime_type[i];

  try
    {
    m_preferred = boost::lexical_cast<double>(val);
    }
  catch(std::bad_cast)
    {
    return false;
    }

  SkipSpaces(i, i_mime_type);
  return (i == i_mime_type.size());
  }
//------------------------------------------------------------------------------
inline
bool MimeType::Matches(const std::string& i_mime_type) const
  {
  std::string::size_type i = 0;

  SkipSpaces(i, i_mime_type);
  if (i == i_mime_type.size())
    return false;

  std::string cur_type;

  for (; i < i_mime_type.size() && !isspace(i_mime_type[i]) && i_mime_type[i] != '/'; ++i)
    cur_type += i_mime_type[i];

  if (m_type != "*" && cur_type != m_type)
    return false;

  if (i == i_mime_type.size() || i_mime_type[i] != '/')
    return false;

  ++i;

  SkipSpaces(i, i_mime_type);
  if (i == i_mime_type.size())
    return false;

  std::string cur_sub_type;

  for (; i < i_mime_type.size() && !isspace(i_mime_type[i]) && i_mime_type[i] != ';'; ++i)
    cur_sub_type += i_mime_type[i];

  return m_sub_type == "*" || cur_sub_type == m_sub_type;
  }
//------------------------------------------------------------------------------
inline
std::string MimeType::AsString(bool i_fl_show_pref) const
  {
  std::stringstream res;
  res << m_type << "/" << m_sub_type;
  
  if (i_fl_show_pref)
    res << ";q=" << m_preferred;

  res << (char)0;

  return res.str();
  }
//------------------------------------------------------------------------------
#endif // MIME_TYPE_H
