/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "plain_storage.h"
#include "plain_position.h"
#include "abstract_memory_mapper.h"
#include <unicode/ucnv.h>

//------------------------------------------------------------------------------
PlainStorage::PlainStorage(const std::string& i_codepage, const std::string& i_mime_type, const AbstractMemoryMapper* ip_storage) throw(StorageException):
  m_codepage(i_codepage), m_mime_type(i_mime_type), mp_storage(ip_storage)
  {
  UErrorCode status = U_ZERO_ERROR;
  mp_converter = ucnv_open(m_codepage.c_str(), &status);

  if (U_FAILURE(status))
    throw StorageException("ConverterCreationFailed", "Failed to create the converter");
  }
//------------------------------------------------------------------------------
PlainStorage::~PlainStorage() throw()
  {
  delete mp_storage;
  ucnv_close(mp_converter);
  }
//------------------------------------------------------------------------------
UnicodeString PlainStorage::Get(const AbstractPosition& i_position) const throw(StorageException)
  {
  const PlainPosition* p_pos = dynamic_cast<const PlainPosition*>(&i_position);
  if (!p_pos)
    throw StorageException("WrongPositionType", "Plain storage expects position of PlainPosition type");

  assert(p_pos -> m_pos + p_pos -> m_length <= mp_storage -> Length());

  UErrorCode status = U_ZERO_ERROR;
  UnicodeString str(mp_storage -> Get() + p_pos -> m_pos, p_pos -> m_length,
		    mp_converter, status);

  if (U_FAILURE(status))
    throw StorageException("ConverionFailed", "Failed to create new string with convertion");

  return str;
  }
//------------------------------------------------------------------------------
