/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "prefix_match_strategy.h"
#include "index_utils.h"
#include "abstract_controller.h"

//------------------------------------------------------------------------------
PrefixMatchStrategy::~PrefixMatchStrategy() throw()
 {
 }
//------------------------------------------------------------------------------
UnicodeString PrefixMatchStrategy::GetStrategyName() const throw()
  {
  return "prefix";
  }
//------------------------------------------------------------------------------
UnicodeString PrefixMatchStrategy::GetStrategyDescription() const throw()
  {
  return "Match words prefixes";
  }
//------------------------------------------------------------------------------
void PrefixMatchStrategy::FindKeyWords(std::list<index_type>& o_keywords, AbstractController& io_controller,
                                       const AbstractIndex& i_index, const UnicodeString& i_word) const throw(StrategyException)
  {
  UErrorCode status = U_ZERO_ERROR;
  CollationKey key_full;

  // We need to produce first a normal collation key to start from ...
  i_index.GetCollator() -> getCollationKey(i_word, key_full, status);

  if (U_FAILURE(status))
    throw StrategyException("CollationKeyCreationFailed", "Failed to create the collation key in prefix matching strategy");

  int32_t len_full;
  const uint8_t* p_buf_full = key_full.getByteArray(len_full);

  // Now we are producing lower bound ...
  int32_t len = i_index.GetCollator() -> getBound(p_buf_full, -1, UCOL_BOUND_LOWER,
						  1, NULL, 0, status);

  if (U_FAILURE(status))
    throw StrategyException("CollationKeyCreationFailed", "Failed to create the collation key in prefix matching strategy");

  uint8_t* p_buf = new uint8_t[len];
  i_index.GetCollator() -> getBound(p_buf_full, -1, UCOL_BOUND_LOWER,
				    1, p_buf, len, status);

  if (U_FAILURE(status))
    {
    delete[] p_buf;
    throw StrategyException("CollationKeyCreationFailed", "Failed to create the collation key in prefix matching strategy");
    }

  CollationKey key_lower(p_buf, len);

  delete[] p_buf;

  // And here goes upper bound.
  len = i_index.GetCollator() -> getBound(p_buf_full, -1, UCOL_BOUND_UPPER_LONG,
					  1, NULL, 0, status);

  if (U_FAILURE(status))
    throw StrategyException("CollationKeyCreationFailed", "Failed to create the collation key in prefix matching strategy");

  p_buf = new uint8_t[len];
  i_index.GetCollator() -> getBound(p_buf_full, -1, UCOL_BOUND_UPPER_LONG,
				    1, p_buf, len, status);

  if (U_FAILURE(status))
    {
    delete[] p_buf;
    throw StrategyException("CollationKeyCreationFailed", "Failed to create the collation key in prefix matching strategy");
    }

  CollationKey key_upper(p_buf, len);

  delete[] p_buf;

  // Now we should use lower and upper bounds to produce the range we need:

  IndexIterator lower = std::lower_bound(IndexIterator(0), IndexIterator(i_index.GetNumberOfKeyWords()), key_lower, IndexItemsComparator(&i_index));
  IndexIterator upper = std::upper_bound(IndexIterator(0), IndexIterator(i_index.GetNumberOfKeyWords()), key_upper, IndexItemsComparator(&i_index));

  for (IndexIterator it = lower; it != upper; ++it)
    {
    if (!io_controller.IsMatchAllowed())
      break;
    io_controller.OneMoreMatch();

    o_keywords.push_back(*it);
    }
  }
//------------------------------------------------------------------------------
