/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "top_levinshtein_match_strategy.h"
#include "abstract_index.h"
#include "approx_string_matcher.h"
#include "abstract_controller.h"

//------------------------------------------------------------------------------
TopLevinshteinMatchStrategy::TopLevinshteinMatchStrategy(ulong i_words_num, bool i_fl_strict, ulong i_max_dist):
  m_words_num(i_words_num), m_fl_strict(i_fl_strict), m_max_dist(i_max_dist)
  {
  }
//------------------------------------------------------------------------------
TopLevinshteinMatchStrategy::~TopLevinshteinMatchStrategy() throw()
 {
 }
//------------------------------------------------------------------------------
UnicodeString TopLevinshteinMatchStrategy::GetStrategyName() const throw()
  {
  return (std::string("lev-top-") + boost::lexical_cast<std::string>(m_words_num)).c_str();
  }
//------------------------------------------------------------------------------
UnicodeString TopLevinshteinMatchStrategy::GetStrategyDescription() const throw()
  {
  return (std::string("Match the top ") + boost::lexical_cast<std::string>(m_words_num) + (" words closest by Levinshtein distance")).c_str();
  }
//------------------------------------------------------------------------------
void TopLevinshteinMatchStrategy::FindKeyWords(std::list<index_type>& o_keywords, AbstractController& io_controller,
                                               const AbstractIndex& i_index, const UnicodeString& i_word) const throw(StrategyException)
  {
  index_type index_size = i_index.GetNumberOfKeyWords();
  ApproxStringMatcher matcher;
  std::priority_queue<KeyWordInfo> queue;

  for (index_type i = 0; i < index_size; ++i)
    {
    if (!io_controller.IsMatchAllowed())
      break;

    UnicodeString keyword = i_index.GetKeyWord(i);
    ulong diff = std::abs(keyword.length() - i_word.length());

    if (m_max_dist && diff > m_max_dist)
      continue;

    if (queue.size() == m_words_num && diff > queue.top().m_dist)
      continue;

    ulong cur_dist = (ulong)matcher.Distance(i_word, i_index.GetKeyWord(i));

    if (m_max_dist && cur_dist > m_max_dist || m_fl_strict && cur_dist >= (ulong)i_word.length())
      continue;

    if (queue.size() < m_words_num)
      {
      queue.push(KeyWordInfo(i, cur_dist));
      io_controller.OneMoreMatch();
      }
    else if (queue.top().m_dist > cur_dist)
      {
      queue.pop();
      queue.push(KeyWordInfo(i, cur_dist));
      }
    }

  while (!queue.empty())
    {
    o_keywords.push_front(queue.top().m_keyword_index);
    queue.pop();
    }
  }
//------------------------------------------------------------------------------
