/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef TOP_LEVINSHTEIN_MATCH_STRATEGY_H
#define TOP_LEVINSHTEIN_MATCH_STRATEGY_H

#include "abstract_strategy.h"

//------------------------------------------------------------------------------
class TopLevinshteinMatchStrategy: public AbstractStrategy
  {
  public:
    TopLevinshteinMatchStrategy(index_type i_words_num, bool i_fl_strict = true,
                                ulong i_max_dist = 0);

    virtual ~TopLevinshteinMatchStrategy() throw();

    UnicodeString GetStrategyName() const throw();

    UnicodeString GetStrategyDescription() const throw();

    void FindKeyWords(std::list<index_type>& o_keywords, AbstractController& io_controller,
                      const AbstractIndex& i_index, const UnicodeString& i_word) const throw(StrategyException);

  private:
    index_type m_words_num;
    bool m_fl_strict;
    ulong m_max_dist;

    struct KeyWordInfo
      {
      index_type m_keyword_index;
      ulong      m_dist;

      KeyWordInfo(index_type i_keyword_index, ulong i_dist):
        m_keyword_index(i_keyword_index), m_dist(i_dist) {}
 
      bool operator < (const KeyWordInfo& i_info) const
        {
        return m_dist < i_info.m_dist;
        }
      };

  };
//------------------------------------------------------------------------------

#endif // TOP_LEVINSHTEIN_MATCH_STRATEGY_H
