/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"

#ifdef DICTDPP_USE_XALAN

#include "xml_transformer.h"

#include <xalanc/XalanTransformer/XalanCompiledStylesheet.hpp>

#include <unicode/ustring.h>

//-----------------------------------------------------------------------------
XMLTransformer::XMLTransformer()
  {
  }
//------------------------------------------------------------------------------
std::string XMLTransformer::GetInputMimeTypeForTransform(const UnicodeString& i_name) const throw(TransformerException)
  {
  if (m_transforms.find(i_name) == m_transforms.end())
    throw TransformerException("UnknownTransform", std::string("Unknown transform name: ") + AsString(i_name));

  return m_transforms.find(i_name) -> second.m_in_mime_type;
  }
//------------------------------------------------------------------------------
std::string XMLTransformer::GetOutputMimeTypeForTransform(const UnicodeString& i_name) const throw(TransformerException)
  {
  if (m_transforms.find(i_name) == m_transforms.end())
    throw TransformerException("UnknownTransform", std::string("Unknown transform name: ") + AsString(i_name));

  return m_transforms.find(i_name) -> second.m_out_mime_type;
  }
//------------------------------------------------------------------------------
void XMLTransformer::AddTransform(const UnicodeString& i_name, const std::string& i_in_mime_type,
				  const std::string& i_out_mime_type, const std::string& i_params) throw(TransformerException)
  {
  if (m_transforms.find(i_name) != m_transforms.end())
    throw TransformerException("DuplicateTransform", "Transform already exists");

  m_transforms[i_name] = TransformInfo(i_in_mime_type, i_out_mime_type, i_params);

  if (m_stylesheets.find(i_params) == m_stylesheets.end())
    {
    if (m_transformer.compileStylesheet(i_params.c_str(), m_stylesheets[i_params]))
      throw TransformerException("CannotPrepareTransform", std::string("Cannot precompile stylesheet ") + i_params + ": " + m_transformer.getLastError());
    }
  }
//------------------------------------------------------------------------------
XMLTransformer::~XMLTransformer() throw()
  {
  }
//------------------------------------------------------------------------------
UnicodeString XMLTransformer::GetTransformerName() const throw()
  {
  return "XML";
  }
//------------------------------------------------------------------------------
UnicodeString XMLTransformer::Transform(const UnicodeString& i_input, const UnicodeString& i_transform) const throw(TransformerException)
  {
  if (m_transforms.find(i_transform) == m_transforms.end())
    throw TransformerException("CannotTransform", std::string("Cannot perform processing: unknown transform requested"));

  char* p_buf = new char[i_input.length()*6 + 1];
  int32_t len;

  UErrorCode status = U_ZERO_ERROR;
  u_strToUTF8(p_buf, i_input.length()*6 + 1, &len, i_input.getBuffer(), i_input.length(), &status);

  assert(len < i_input.length()*6 + 1);

  std::ostringstream out;
  std::stringstream in;
  in << "<?xml version=\"1.0\" encoding=\"utf-8\"?>" << std::endl;
  in << "<entry>" << std::endl;
  in << p_buf;
  in << "</entry>" << std::endl;

  xalanc::XalanTransformer transformer;
  transformer.setOutputEncoding(xalanc::XalanDOMString("utf-8"));
  if (transformer.transform(in, m_stylesheets.find(m_transforms.find(i_transform) -> second.m_file_name) -> second, &out))
    {
    delete[] p_buf;
    throw TransformerException("CannotTransform", std::string("Cannot perform transformation: ") + transformer.getLastError());
    }

  delete[] p_buf;

  std::string str = out.str();
  return UnicodeString(str.c_str(), str.length(), "utf-8");
  }
//------------------------------------------------------------------------------
#endif // DICTDPP_ENABLE_XALAN
