/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef XML_TRANSFORMER_H
#define XML_TRANSFORMER_H

#ifdef DICTDPP_USE_XALAN

#include "abstract_transformer.h"

#include <xalanc/XalanTransformer/XalanTransformer.hpp>

//------------------------------------------------------------------------------
class XMLTransformer: public AbstractTransformer
  {
  public:
    XMLTransformer();

    ~XMLTransformer() throw();

    virtual UnicodeString GetTransformerName() const throw();

    virtual std::string GetInputMimeTypeForTransform(const UnicodeString& i_transform) const throw(TransformerException);

    virtual std::string GetOutputMimeTypeForTransform(const UnicodeString& i_transform) const throw(TransformerException);

    virtual void AddTransform(const UnicodeString& i_name, const std::string& i_in_mime_type,
			      const std::string& i_out_mime_type, const std::string& i_params) throw(TransformerException);

    virtual UnicodeString Transform(const UnicodeString& i_input, const UnicodeString& i_transform) const throw(TransformerException);

  private:
    struct TransformInfo
      {
      std::string m_in_mime_type, m_out_mime_type, m_file_name;

      TransformInfo(const std::string& i_in_mime_type, const std::string& i_out_mime_type,
		    const std::string& i_file_name):
        m_in_mime_type(i_in_mime_type), m_out_mime_type(i_out_mime_type),
        m_file_name(i_file_name) {}

      TransformInfo() {}
      };

    /// Maps transform name to transform info
    std::map<UnicodeString, TransformInfo> m_transforms;
    /// Maps stylesheet file name to compiled stylesheet
    std::map<std::string, const xalanc::XalanCompiledStylesheet*> m_stylesheets;
    xalanc::XalanTransformer m_transformer; // to hold compiled stylesheets
  };
//------------------------------------------------------------------------------

#endif // DICTDPP_ENABLE_XALAN

#endif // XML_TRANSFORMER_H
