/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef CONTROL_THREAD_H
#define CONTROL_THREAD_H

#include <time.h>
//------------------------------------------------------------------------------
class ControlThread
  {
  friend class Server;

  public:
    bool IsShuttingDown() const;

    time_t GetCurrentTime() const;

    void operator ()();

  private:
    // Yes, a bit of hack :-(((
    // I don't use locks to control access to these variables as it would mean
    // quite high overhead. Typically these variables should occupy
    // one word on most systems, so compiler should generate code that
    // modifies each of them in one instruction, so they should not be
    // in a broken stake at any time. However, if for some system it's not true,
    // it's possible to add locks for that particular system ...
    time_t m_curr_time, m_timeout;
    bool m_fl_shutting_down;

  private:
    ControlThread(time_t i_timeout): m_timeout(i_timeout), m_fl_shutting_down(false)
      {
      time(&m_curr_time);
      }

    void SetIsShuttingDown() { m_fl_shutting_down = true; }
  };
//------------------------------------------------------------------------------
inline
bool ControlThread::IsShuttingDown() const
  {
  return m_fl_shutting_down;
  }
//------------------------------------------------------------------------------
inline
time_t ControlThread::GetCurrentTime() const
  {
  return m_curr_time;
  }
//------------------------------------------------------------------------------
struct ControlThreadWrapper
  {
  ControlThread* mp_thread;

  ControlThreadWrapper(ControlThread* ip_thread): mp_thread(ip_thread) {}

  void operator() () { return mp_thread -> operator ()(); }
  };
//--------------------------------------------------------------------

#endif // CONTROL_THREAD_H
