/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "controller.h"
#include "control_thread.h"

//------------------------------------------------------------------------------
QueryControlOptions::QueryControlOptions()
  {
  m_max_match_time = std::numeric_limits<time_t>::max();
  m_max_define_time = std::numeric_limits<time_t>::max();
  m_max_match_entries = std::numeric_limits<index_type>::max();
  m_max_define_entries = std::numeric_limits<index_type>::max();
  }
//------------------------------------------------------------------------------
void Controller::StartMatch()
  {
  m_cur_match_entries = 0;
  m_fl_match_interrupted = false;
  if (m_options.m_max_match_time != (time_t)std::numeric_limits<time_t>::max())
    m_cur_max_match_time = time(NULL) + m_options.m_max_match_time;
  }
//------------------------------------------------------------------------------
void Controller::StartDefine()
  {
  m_cur_define_entries = 0;
  m_fl_define_interrupted = false;
  if (m_options.m_max_define_time != std::numeric_limits<time_t>::max())
    m_cur_max_define_time = time(NULL) + m_options.m_max_define_time;
  }
//------------------------------------------------------------------------------
void Controller::OneMoreMatch()
  {
  ++m_cur_match_entries;
  }
//------------------------------------------------------------------------------
void Controller::OneMoreDefine()
  {
  ++m_cur_define_entries;
  }
//------------------------------------------------------------------------------
bool Controller::IsMatchAllowed()
  {
  if (mp_ctrl_thread -> IsShuttingDown())
    {
    m_fl_match_interrupted = true;
    return false;
    }

  if (m_options.m_max_match_entries != std::numeric_limits<index_type>::max() &&
      m_cur_match_entries >= m_options.m_max_match_entries)
    {
    m_fl_match_interrupted = true;
    return false;
    }

  if (m_options.m_max_match_time != std::numeric_limits<time_t>::max() &&
      mp_ctrl_thread -> GetCurrentTime() >= m_cur_max_match_time)
    {
    m_fl_match_interrupted = true;
    return false;
    }

  return true;
  }
//------------------------------------------------------------------------------
bool Controller::IsDefineAllowed()
  {
  if (mp_ctrl_thread -> IsShuttingDown())
    {
    m_fl_define_interrupted = true;
    return false;
    }

  if (m_options.m_max_define_entries != std::numeric_limits<index_type>::max() &&
      m_cur_define_entries >= m_options.m_max_define_entries)
    {
    m_fl_define_interrupted = true;
    return false;
    }

  if (m_options.m_max_define_time != std::numeric_limits<time_t>::max() &&
      mp_ctrl_thread -> GetCurrentTime() >= m_cur_max_define_time)
    {
    m_fl_define_interrupted = true;
    return false;
    }

  return true;
  }
//------------------------------------------------------------------------------
bool Controller::WasMatchInterrupted()
  {
  return m_fl_match_interrupted;
  }
//------------------------------------------------------------------------------
bool Controller::WasDefineInterrupted()
  {
  return m_fl_define_interrupted;
  }
//------------------------------------------------------------------------------
