/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "server.h"
#include "control_thread.h"
#include "dictd++.h"

//------------------------------------------------------------------------------
Server* gp_server = NULL;
//------------------------------------------------------------------------------
#ifdef _WIN32
  std::string g_config_path;
  std::string g_log_path;
#else
  std::string g_config_path = "dictd++.xml";
  std::string g_log_path    = "/var/log/DictD++.log";
#endif
//------------------------------------------------------------------------------
#ifdef _WIN32
//------------------------------------------------------------------------------
void LoadInfoFromRegistry()
  {
  HKEY h_key;

  LONG res = RegOpenKeyEx(HKEY_CURRENT_USER, "SOFTWARE\\DictD++", 0, KEY_QUERY_VALUE, &h_key);

  if (res != ERROR_SUCCESS)
    res = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\DictD++", 0, KEY_QUERY_VALUE, &h_key);

  if (res == ERROR_SUCCESS)
    {
    DWORD buf_len;

    if (RegQueryValueEx(h_key, "ConfigPath", NULL, NULL, NULL, &buf_len) == ERROR_SUCCESS)
      {
      char* p_buf = new char[buf_len];

      RegQueryValueEx(h_key, "ConfigPath", NULL, NULL, (LPBYTE)p_buf, &buf_len);
      g_config_path = std::string(p_buf);

      delete[] p_buf;
      }

    if (RegQueryValueEx(h_key, "LogPath", NULL, NULL, NULL, &buf_len) == ERROR_SUCCESS)
      {
      char* p_buf = new char[buf_len];

      RegQueryValueEx(h_key, "LogPath", NULL, NULL, (LPBYTE)p_buf, &buf_len);
      g_log_path = std::string(p_buf);

      delete[] p_buf;
      }

    RegCloseKey(h_key);
    }
  }
//------------------------------------------------------------------------------
BOOL CtrlHandler(DWORD fdwCtrlType) 
  { 
  switch (fdwCtrlType) 
    { 
    case CTRL_C_EVENT:           // Handle the CTRL+C signal.
    case CTRL_CLOSE_EVENT:       // CTRL+CLOSE: confirm that the user wants to exit. 
    case CTRL_BREAK_EVENT: 
    case CTRL_LOGOFF_EVENT: 
    case CTRL_SHUTDOWN_EVENT: 
      if (gp_server)
	{
	g_log << INFO << "Starting to shutdown, timeout is " << gp_server -> GetTimeAccuracy() << " seconds" << endlog;
	gp_server -> SetIsShuttingDown();
	}
      return TRUE;
  
    default: 
      g_log << WARN << "Unknown control signal received, passing to the default handler" << endlog;
      return FALSE; 
    } 
} 
//------------------------------------------------------------------------------
SERVICE_STATUS          service_status; 
SERVICE_STATUS_HANDLE   hStatus; 
//------------------------------------------------------------------------------
void ControlHandler(DWORD request) 
  { 
  switch (request) 
    { 
    case SERVICE_CONTROL_STOP: 
    case SERVICE_CONTROL_SHUTDOWN: 
      g_log << INFO << "Starting to shutdown, timeout is " << gp_server -> GetTimeAccuracy() << " seconds" << endlog;
      gp_server -> SetIsShuttingDown();

      service_status.dwWin32ExitCode = 0;
      service_status.dwCurrentState = SERVICE_STOP_PENDING;
      SetServiceStatus (hStatus, &service_status);
      return; 
	
    default:
       break;
    } 
 
  // Report current status
  SetServiceStatus (hStatus, &service_status);
 
  return; 
  }
//------------------------------------------------------------------------------
void ServiceMain(int argc, char* argv[])
  { 
  service_status.dwServiceType = SERVICE_WIN32; 
  service_status.dwCurrentState = SERVICE_START_PENDING; 
  service_status.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN;
  service_status.dwWin32ExitCode = 0; 
  service_status.dwServiceSpecificExitCode = 0; 
  service_status.dwCheckPoint = 0; 
  service_status.dwWaitHint = 0; 
 
  hStatus = RegisterServiceCtrlHandler("DictD++", (LPHANDLER_FUNCTION)ControlHandler);

  if (hStatus == (SERVICE_STATUS_HANDLE)0) 
    { 
    // Registering Control Handler failed
    std::cerr << "Cannot register service, terminating" << std::endl;
    return;
    }

  // Initialize Service
  try
    {
    Initialize();
    gp_server = new Server;
    gp_server -> ParseConfigAndInit(g_config_path);
    }
  catch(const GeneralException& ex)
    {
    g_log << CRIT << ex << endlog;
    g_log << CRIT << "Cannot handle this exception, terminating server" << endlog;
    service_status.dwCurrentState = SERVICE_STOPPED; 
    service_status.dwWin32ExitCode = -1; 
    SetServiceStatus(hStatus, &service_status);
    return; 
    }
  catch(...)
    {
    g_log << CRIT << "Unknown exception, terminating server" << endlog;
    service_status.dwCurrentState = SERVICE_STOPPED; 
    service_status.dwWin32ExitCode = -1; 
    SetServiceStatus(hStatus, &service_status);
    return; 
    }

  service_status.dwCurrentState = SERVICE_RUNNING; 
  SetServiceStatus (hStatus, &service_status);

  try
    {
    gp_server -> Run();
    }
  catch(...)
    {
    g_log << CRIT << "Unknown exception, terminating server" << endlog;

    service_status.dwWin32ExitCode = 0;
    service_status.dwCurrentState = SERVICE_STOP_PENDING;
    SetServiceStatus (hStatus, &service_status);

    delete gp_server;
    DeInitialize();

    service_status.dwWin32ExitCode = 0;
    service_status.dwCurrentState = SERVICE_STOPPED;
    SetServiceStatus (hStatus, &service_status);

    return;
    }

  delete gp_server;
  DeInitialize();

  service_status.dwWin32ExitCode = 0;
  service_status.dwCurrentState = SERVICE_STOPPED;
  SetServiceStatus (hStatus, &service_status);

  return; 
  }
//------------------------------------------------------------------------------
int main(int argc, char* argv[])
try
  {
#if defined(_DEBUG) && defined(_MSC_VER)
  _CrtSetDbgFlag (_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
#endif
  //_crtBreakAlloc = 6737;

  if (argc > 2 || argc == 2 && strcmp(argv[1], "-n") && strcmp(argv[1], "--no-detach"))
    {
    std::cerr << "Usage:\nDictD++ [options]\nWhere options are:\n\n"
		 "-n, --no-detach        Start as usual process and do not detach from the console\n";
    return 1;
    }

  bool fl_as_service = true;

  if (argc == 2 && (!strcmp(argv[1], "-n") || !strcmp(argv[1], "--no-detach")))
    fl_as_service = false;

  LoadInfoFromRegistry();

  std::ofstream* p_ofs = new std::ofstream(g_log_path.c_str());

  if (!p_ofs -> is_open())
    {
    std::cerr << "FATAL: log path is unknown or incorrect, please reinstall application" << std::endl;
    std::cerr << "(or specify path to log file in registry key \'SOFTWARE\\DictD++\\LogPath\'" << std::endl;
    std::cerr << " if you know what it is all about ;-)" << std::endl;

    return 2;
    }

  g_log.SetOutput(*p_ofs, true);
  g_log.SetLevel(DEBUG);
  g_log.SetAddTime(true);
  g_log.SetAddLevel(true);

  if (!fl_as_service)
    {
    g_log.AddOutput(std::clog, false);
    g_log.SetLevel(INFO);
    g_log.SetAddTime(false);
    g_log.SetAddLevel(true);
    }

  g_log << INFO << "Logging has been started" << endlog;

  if (g_config_path.empty())
    {
    g_log << CRIT << "Config path is unknown, please reinstall application" << endlog;
    g_log << CRIT << "(or specify path to config file in registry key \'SOFTWARE\\DictD++\\ConfigPath\'" << endlog;
    g_log << CRIT << " if you know what it is all about ;-)"<< endlog;
    return 2;
    }

  if (fl_as_service)
    {
    SERVICE_TABLE_ENTRY service_table[2];

    service_table[0].lpServiceName = "DictD++";
    service_table[0].lpServiceProc = (LPSERVICE_MAIN_FUNCTION)ServiceMain;

    service_table[1].lpServiceName = NULL;
    service_table[1].lpServiceProc = NULL;

    if (!StartServiceCtrlDispatcher(service_table))
      {
      g_log.AddOutput(std::clog, false);
      g_log.SetLevel(INFO);
      g_log.SetAddTime(false);
      g_log.SetAddLevel(true);
  
      g_log << CRIT << "StartServiceCtrlDispatcher call failed, cannot start the service" << endlog;
      g_log << CRIT << "You should use \'Services\' tool to start the service" << endlog;
      g_log << CRIT << "Starting in console mode" << endlog;
      }
    else
      return 0;
    }

  BOOL fl_success = SetConsoleCtrlHandler((PHANDLER_ROUTINE) CtrlHandler, TRUE);

  if (!fl_success)
    {
    g_log << CRIT << "Was not able to set control handler, terminating" << endlog;
    return 1;
    }

  try
    {
    Initialize();

    //TestSuite test_suite(g_log);

    gp_server = new Server;
    gp_server -> ParseConfigAndInit(g_config_path);
    gp_server -> Run();
    }
  catch(const GeneralException& ex)
    {
    g_log << CRIT << ex << endlog;
    g_log << CRIT << "Cannot handle this exception, terminating server" << endlog;
    }
  catch(...)
    {
    g_log << CRIT << "Unknown exception, terminating server" << endlog;
    }

  delete gp_server;
  DeInitialize();

  return 0;
  }
catch(...)
  {
  std::cerr << "Unknown exception at the top level, please submit bug report to the author" << std::endl;
  std::cerr << "Was not able to terminate the server gracefully" << std::endl;
  std::cerr << "Re-throwing the exeption ... " << std::endl;
  throw;
  }
//------------------------------------------------------------------------------
#else

#include <boost/program_options.hpp>

#include <signal.h>
#include <sys/types.h>
#include <unistd.h>

void dictd_signal_handler(int sig)
  {
  switch (sig)
    {
    case SIGINT:
    case SIGTERM:
      g_log << INFO << "Termination signal received" << endlog;
      g_log << INFO << "Starting to shutdown, timeout is " << gp_server -> GetTimeAccuracy() << " seconds" << endlog;
      gp_server -> SetIsShuttingDown();
      break;

    default:
      g_log << CRIT << "Unexpected signal " << sig << " caught" << endlog;
    }
  }

int daemonize()
  {
  pid_t pid = -1;
  pid = fork();

  if (pid > 0)
    return 0;

  // TODO: ssid, umask, ... ?

  close(STDIN_FILENO);
  close(STDOUT_FILENO);
  close(STDERR_FILENO);

  return 1;
  }

namespace po = boost::program_options;

int main(int argc, char* argv[])
  {
  po::options_description desc("Usage");
  desc.add_options()
    ("help,h", "Display usage information")
    ("no-detach,n", "No detach")
    ("log-file,l", po::value<std::string>(), "Log file name")
    ("config-file,c", po::value<std::string>(), "Config file name");

  po::variables_map vm;
  po::store(po::parse_command_line(argc, argv, desc), vm);
  po::notify(vm);

  if (vm.count("help"))
    {
    std::cout << desc << std::endl;
    return 1;
    }

  std::ofstream log_file;

  if (vm.count("log-file"))
    {
    std::string fname(vm["log-file"].as<std::string>());
    log_file.open(fname.c_str(), std::ios::app);

    if (!log_file)
      {
      std::cerr << "Cannot open file " << fname << ", terminating" << std::endl;
      return 2;
      }

    g_log.SetOutput(log_file, false);
    }
  else
    {
    if (!vm.count("no-detach"))
      std::clog << "WARNING: log file name was not set and no-detach option was not specified." << std::endl
                << "All logging output will be lost." << std::endl;
    else
      {
      std::clog << "Log file name was not set, standard output will be used instead." << std::endl;
      g_log.SetOutput(std::clog, false);
      }
    }

//  g_log.SetLevel(DEBUG);
  g_log.SetLevel(TRACE);
  g_log.SetAddTime(true);
  g_log.SetAddLevel(true);
  g_log << INFO << "Logging has been started" << endlog;

  signal(SIGINT, &dictd_signal_handler);
  signal(SIGTERM, &dictd_signal_handler);

  if (vm.count("no-detach"))
    g_log << TRACE << "Not detaching from terminal" << endlog;
  else
    {
    switch(daemonize())
      {
      case 0:
        g_log << TRACE << "Exiting from parent" << endlog;
        return 0;
      
      case 1:
        g_log << TRACE << "Continuing execution in child" << endlog;
        break;

      default:
        g_log << CRIT << "Daemonizing error, terminating" << endlog;
        return 3;
      }
    }

  try
    {
    Initialize();

    gp_server = new Server;
    
    if (vm.count("config-file"))
      gp_server -> ParseConfigAndInit(vm["config-file"].as<std::string>());
    else
      {
      g_log << TRACE << "Config file name was not specified, using default name " << g_config_path << endlog;
      gp_server -> ParseConfigAndInit(g_config_path);
      }
    
    gp_server -> Run();
    }
  catch(const GeneralException& ex)
    {
    g_log << CRIT << ex << endlog;
    g_log << CRIT << "Cannot handle this exception, terminating server" << endlog;
    }
  catch(...)
    {
    g_log << CRIT << "Unknown exception, terminating server" << endlog;
    }

  delete gp_server;
  DeInitialize();
  }
//------------------------------------------------------------------------------
#endif
