/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "security_manager.h"
#include <dictd_utils.h>
#include <session_info.h>
#include "server.h"

#ifndef _WIN32
#include <netdb.h>
#include <arpa/inet.h>
#endif

//------------------------------------------------------------------------------
NetAccess::NetAccess(const std::string& i_ip, const std::string& i_mask, bool i_fl_allowed):
  m_fl_allowed(i_fl_allowed)
  {
  m_ip = inet_addr(i_ip.c_str());
  if (m_ip == INADDR_NONE)
    {
    hostent* p_host_info = gethostbyname(i_ip.c_str());
    if (!p_host_info)
      throw SecurityManagerException("CannotParseAddress", "Cannot parse address " + i_ip);

    // Mmm??? $AT
    m_ip = *(ulong*)p_host_info -> h_addr_list[0];
    }

  m_mask = inet_addr(i_mask.c_str());

  if (m_mask == INADDR_NONE)
    m_mask = 0xFFFFFFFF;
  //throw SecurityManagerException("CannotParseAddress", "Cannot parse network mask " + i_mask);
  }
//------------------------------------------------------------------------------
AccessInfo::AccessInfo()
  {
  m_fl_default_access_policy = true;
  }
//------------------------------------------------------------------------------
bool AccessInfo::IsConnectionAllowed(const Socket& i_socket) const
  {
  uint32_t addr = i_socket.GetPeerAddress();

  for (std::list<NetAccess>::const_iterator it = m_net_access_list.begin();
       it != m_net_access_list.end(); ++it)
    if (it -> m_ip == (addr & it -> m_mask))
      return it -> m_fl_allowed;

  return m_fl_default_access_policy;
  }
//------------------------------------------------------------------------------
AbstractController* SecurityManager::CreateController(const SessionInfo& i_session_info) const
  {
  assert (m_users.find(i_session_info.m_user) != m_users.end());

  const AccessInfo& info = m_users.find(i_session_info.m_user) -> second.m_access_info;
  Controller* p_controller = new Controller(&m_server.GetControlThread());

  p_controller -> m_options = info.m_query_ctrl_options;

  return p_controller;
  }
//------------------------------------------------------------------------------
bool SecurityManager::IsAuthenticated(const std::string& i_msg_id, const std::string& i_user, const std::string& i_key) const
  {
  if (m_users.find(i_user) == m_users.end())
    return false;

  g_log << DEBUG << "Our key is " << Tools::GetMD5(i_msg_id + m_users.find(i_user) -> second.m_password) << endlog;

  return Tools::GetMD5(i_msg_id + m_users.find(i_user) -> second.m_password) == i_key;
  }
//------------------------------------------------------------------------------
bool SecurityManager::IsConnectionAllowed(const Socket& i_socket, const std::string& i_user) const
  {
  if (m_users.find(i_user) == m_users.end())
    return false;

  return m_users.find(i_user) -> second.m_access_info.IsConnectionAllowed(i_socket);
  }
//------------------------------------------------------------------------------
std::string SecurityManager::GetUserGroup(const std::string& i_user) const
  {
  assert(m_users.find(i_user) != m_users.end());
  return m_users.find(i_user) -> second.m_group;
  }
//------------------------------------------------------------------------------
void SecurityManager::Validate()
  {
  for (std::map<std::string, UserInfo>::const_iterator it = m_users.begin();
       it != m_users.end(); ++it)
    if (m_groups.find(it -> second.m_group) == m_groups.end())
      throw SecurityManagerException("GroupDoesNotExist", "There is no declaration of the group \"" + it -> second.m_group + "\" that is referenced in declaration of user \"" + it -> first + "\"");

  if (m_groups.find("unknown") == m_groups.end())
    {
    g_log << ERR << "Required group \"unknown\" was not found: adding this group with unlimited access" << endlog;
    m_groups["unknown"].m_fl_default_access_policy = true;
    }

  if (m_users.find("unknown") == m_users.end())
    {
    g_log << ERR << "Required user \"unknown\" was not found: adding this user with unlimited access" << endlog;
    m_users["unknown"].m_group = "unknown";
    m_users["unknown"].m_access_info.m_fl_default_access_policy = true;
    }
  }
//------------------------------------------------------------------------------
