/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef SECURITY_MANAGER_H
#define SECURITY_MANAGER_H

#include "controller.h"

//--------------------------------------------------------------------
// This is an exception class used to report errors
// for all security manager operations.
class SecurityManagerException: public GeneralException
  {
  public:
    SecurityManagerException(const std::string& i_err_id,
		             const std::string& i_msg):
      GeneralException("SecurityManager", i_err_id, i_msg) {}
  };
//------------------------------------------------------------------------------
class SessionInfo;
class Socket;
class Server;
//------------------------------------------------------------------------------
struct NetAccess
  {
  uint32_t m_ip, m_mask;
  bool m_fl_allowed;
  NetAccess(const std::string& i_ip, const std::string& i_mask, bool i_fl_allowed);
  };
//------------------------------------------------------------------------------
struct AccessInfo
  {
  bool m_fl_default_access_policy;
  std::list<NetAccess> m_net_access_list;
  QueryControlOptions m_query_ctrl_options;

  bool IsConnectionAllowed(const Socket& i_socket) const;

  AccessInfo();
  };
//------------------------------------------------------------------------------
struct UserInfo
  {
  std::string m_group, m_password;
  AccessInfo m_access_info;
  };
//------------------------------------------------------------------------------
// This class holds info on users, groups and
// associated permitions, and answers appropriate queries.
//
class SecurityManager
  {
  friend class Server;
  friend class UsersHandler;
  friend class GroupsHandler;

  public:
    AbstractController* CreateController(const SessionInfo& i_session_info) const;

    bool IsAuthenticated(const std::string& i_msg_id, const std::string& i_user, const std::string& i_key) const;

    bool IsConnectionAllowed(const Socket& i_socket, const std::string& i_user) const;

    std::string GetUserGroup(const std::string& i_user) const;

    void Validate();

  private:
    const Server& m_server;
    std::map<std::string, UserInfo> m_users;
    std::map<std::string, AccessInfo> m_groups;

    SecurityManager(const Server& i_server): m_server(i_server) {}
  };
//------------------------------------------------------------------------------

#endif // SECURITY_MANAGER_H

