/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef SERVER_H
#define SERVER_H

#include <xercesc/sax2/DefaultHandler.hpp>
#include <xercesc/sax2/SAX2XMLReader.hpp>

#include <boost/thread/mutex.hpp>
#include <boost/thread/thread.hpp>

#include <dictionary_list.h>
#include "server_thread.h"

#include "security_manager.h"

//--------------------------------------------------------------------
class AbstractIndex;
class AbstractStorage;
class ControlThread;
//--------------------------------------------------------------------
// This is an exception class used to report errors
// for all server operations.
class ServerException: public GeneralException
  {
  public:
    ServerException(const std::string& i_err_id,
		    const std::string& i_msg):
      GeneralException("Server", i_err_id, i_msg) {}
  };
//--------------------------------------------------------------------
class ServerXMLErrorHandler: public xercesc::DefaultHandler
  {
  void error(const xercesc::SAXParseException& ex);
  };
//--------------------------------------------------------------------
class Server
  {
  friend class ServerThread;
  friend class ConfigHandlerBase;
  friend class ServerHandler;

  public:
    Server();
    
    ~Server();

    void ParseConfigAndInit(const std::string& i_file_name) throw (ServerException);

    void Run() throw();

    long GetTimeAccuracy() const;

    const DictionaryList& GetDictionaryList() const;

    SecurityManager& GetSecurityManager() const;

    const ControlThread& GetControlThread() const;

    void SetIsShuttingDown() throw();

    std::string GetCachePath() const;

  private:
    UnicodeString GetStatistics() const throw();

    typedef std::map<ServerThread*, boost::thread*> ThreadContainer;

    // General server info:
    DictionaryList m_dictionary_list;
    SecurityManager* mp_security_manager;
    ThreadContainer m_threads;
    unsigned m_port, m_max_threads;
    std::string m_cache_path;
    UnicodeString m_server_info;
    long m_time_accuracy;
    ControlThread* mp_control_thread;

    // Thread-specific info:
    mutable boost::mutex m_mutex;

    // Parsering-related info:
    xercesc::SAX2XMLReader* mp_parser;
    std::list<ConfigHandlerBase*> m_handlers, m_handlers2remove;
  };
//--------------------------------------------------------------------
inline
std::string Server::GetCachePath() const
  {
  return m_cache_path;
  }
//--------------------------------------------------------------------
inline
const DictionaryList& Server::GetDictionaryList() const
  {
  return m_dictionary_list;
  }
//--------------------------------------------------------------------
inline
long Server::GetTimeAccuracy() const
  {
  return m_time_accuracy;
  }
//--------------------------------------------------------------------
inline
const ControlThread& Server::GetControlThread() const
  {
  return *mp_control_thread;
  }
//--------------------------------------------------------------------
inline
SecurityManager& Server::GetSecurityManager() const
  {
  return *mp_security_manager;
  }
//--------------------------------------------------------------------

#endif // SERVER_H
