/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef SERVER_CONFIG_H
#define SERVER_CONFIG_H

#include <xercesc/sax2/DefaultHandler.hpp>
#include <xercesc/sax2/Attributes.hpp>

//--------------------------------------------------------------------
class AbstractIndex;
class AbstractStorage;
class AbstractStrategy;
class AbstractTransformer;
struct AccessRights;
struct AccessInfo;
struct UserInfo;
class Convertion;
class DictionaryList;
class Server;
//--------------------------------------------------------------------
class ConfigHandlerBase: public xercesc::DefaultHandler
  {
  protected:
    UnicodeString m_text;
    std::map<std::string, UnicodeString> m_attributes;
    Server* mp_server;

  public:
    void ClearHandlersToRemove();

    ConfigHandlerBase* GetParent();

  protected:
    ConfigHandlerBase(Server* ip_server): mp_server(ip_server) {}

    virtual void startElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname,
			      xercesc::Attributes& attributes);

    virtual void characters(const XMLCh* const chars, const unsigned int length);

    virtual void endElement(const XMLCh* const uri, const XMLCh* const localname, const XMLCh* const qname);

    void SetNewHandler(ConfigHandlerBase* ip_new_handler);

    void PopHandler();

    DictionaryList& GetDictionaryList();

    void CopyAttributes(const xercesc::Attributes& i_attributes);
  };
//--------------------------------------------------------------------
class ServerXMLContentHandler: public ConfigHandlerBase
  {
  public:
  ServerXMLContentHandler(Server* ip_server): ConfigHandlerBase(ip_server) {}
  
  void startElement(const XMLCh* const uri, const XMLCh* const localname,
		    const XMLCh* const qname, const xercesc::Attributes& attributes);

  void endElement(const XMLCh* const uri, const XMLCh* const localname,
		  const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class ConfigHandler: public ConfigHandlerBase
  {
  public:
    ConfigHandler(Server* ip_server): ConfigHandlerBase(ip_server) {}

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		      const XMLCh* const qname, const xercesc::Attributes& attributes);
  };
//--------------------------------------------------------------------
class ServerHandler: public ConfigHandlerBase
  {
  public:
    ServerHandler(Server* ip_server): ConfigHandlerBase(ip_server) {}

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		    const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class TransformerListHandler: public ConfigHandlerBase
  {
  public:
    TransformerListHandler(Server* ip_server): ConfigHandlerBase(ip_server) {}

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		      const XMLCh* const qname, const xercesc::Attributes& attributes);
  };
//--------------------------------------------------------------------
class TransformerHandler: public ConfigHandlerBase
  {
  public:
    AbstractTransformer* mp_transformer;

    TransformerHandler(Server* ip_server);

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		      const XMLCh* const qname, const xercesc::Attributes& attributes);

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		    const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class TransformHandler: public ConfigHandlerBase
  {
  UnicodeString m_name;
  std::string m_in_mime_type, m_out_mime_type, m_params;

  public:
    TransformHandler(Server* ip_server): ConfigHandlerBase(ip_server) {}

  void endElement(const XMLCh* const uri, const XMLCh* const localname,
		  const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class StrategyListHandler: public ConfigHandlerBase
  {
  public:
    StrategyListHandler(Server* ip_server): ConfigHandlerBase(ip_server) {}

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		      const XMLCh* const qname, const xercesc::Attributes& attributes);
  };
//--------------------------------------------------------------------
class StrategyHandler: public ConfigHandlerBase
  {
  public:
    UnicodeString m_name, m_descr;
    AbstractStrategy* mp_strategy;
    AccessRights* mp_access_rights;

    StrategyHandler(Server* ip_server);

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
                      const XMLCh* const qname, const xercesc::Attributes& attributes);

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		                const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class DictionaryListHandler: public ConfigHandlerBase
  {
  public:
    DictionaryListHandler(Server* ip_server): ConfigHandlerBase(ip_server) {}

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		                  const XMLCh* const qname, const xercesc::Attributes& attributes);
  };
//--------------------------------------------------------------------
class DictionaryHandler: public ConfigHandlerBase
  {
  public:
    UnicodeString m_dict_name, m_dict_descr;
    std::string m_dict_def_mime_type;
    AbstractIndex* mp_index;
    AbstractStorage* mp_storage;
    std::list<Convertion*>* mp_convertions_list;
    AccessRights* mp_access_rights;

    DictionaryHandler(Server* ip_server);

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		                  const XMLCh* const qname, const xercesc::Attributes& attributes);

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		                const XMLCh* const qname);

    void HandleIndex(const std::string& i_name);

    void HandleBody(const std::string& i_name);

    std::string ProduceCacheName(const std::string& i_name) const;
  };
//--------------------------------------------------------------------
class AllowedConvertionsHandler: public ConfigHandlerBase
  {
  public:
    AllowedConvertionsHandler(Server* ip_server): ConfigHandlerBase(ip_server) {}

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		                  const XMLCh* const qname, const xercesc::Attributes& attributes);
  };
//--------------------------------------------------------------------
class ConvertionHandler: public ConfigHandlerBase
  {
  public:
    Convertion* mp_convertion;

    ConvertionHandler(Server* ip_server);

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		                  const XMLCh* const qname, const xercesc::Attributes& attributes);

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		                const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class DictionarySetHandler: public ConfigHandlerBase
  {
  UnicodeString m_name, m_descr;
  std::list<UnicodeString> m_items;

  public:
    DictionarySetHandler(Server* ip_server): ConfigHandlerBase(ip_server) {}

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		                const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class GroupsHandler: public ConfigHandlerBase
  {
  public:
    GroupsHandler(Server* ip_server): ConfigHandlerBase(ip_server) {}

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
                      const XMLCh* const qname, const xercesc::Attributes& attributes);

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		                const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class GroupHandler: public ConfigHandlerBase
  {
  public:
    AccessInfo* mp_access_info;

    GroupHandler(Server* ip_server, AccessInfo* ip_access_info);

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		                  const XMLCh* const qname, const xercesc::Attributes& attributes);

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		                const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class UsersHandler: public ConfigHandlerBase
  {
  public:
    UsersHandler(Server* ip_server): ConfigHandlerBase(ip_server) {}

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
                      const XMLCh* const qname, const xercesc::Attributes& attributes);

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		                const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class UserHandler: public ConfigHandlerBase
  {
  public:
    UserInfo* mp_user_info;

    UserHandler(Server* ip_server, UserInfo* p_user_info);

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		                  const XMLCh* const qname, const xercesc::Attributes& attributes);

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		                const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class AccessHandler: public ConfigHandlerBase
  {
  public:
    AccessRights* mp_access_rights;

    AccessHandler(Server* ip_server, AccessRights* iop_access_rights);

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		                const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class NetAccessHandler: public ConfigHandlerBase
  {
  public:
    AccessInfo* mp_access_info;

    NetAccessHandler(Server* ip_server, AccessInfo* iop_access_info);

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		                  const XMLCh* const qname, const xercesc::Attributes& attributes);

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		                const XMLCh* const qname);
  };
//--------------------------------------------------------------------
class QueryOptionsHandler: public ConfigHandlerBase
  {
  public:
    AccessInfo* mp_access_info;

    QueryOptionsHandler(Server* ip_server, AccessInfo* iop_access_info);

    void startElement(const XMLCh* const uri, const XMLCh* const localname,
		                  const XMLCh* const qname, const xercesc::Attributes& attributes);

    void endElement(const XMLCh* const uri, const XMLCh* const localname,
		                const XMLCh* const qname);
  };
//--------------------------------------------------------------------

#endif // SERVER_CONFIG_H
