/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "server_thread.h"
#include "server.h"
#include "control_thread.h"

#include <time.h>

#include <dictd_utils.h>

#include <boost/thread/mutex.hpp>

//------------------------------------------------------------------------------
ServerThread::ServerThread(const Socket& i_sock, Server& i_server):
                m_socket(i_sock), m_server(i_server)
  {
  }
//------------------------------------------------------------------------------
ServerThread::~ServerThread()
  {
  }
//------------------------------------------------------------------------------
void ServerThread::ProcessCommand(std::vector<std::string>& i_list)
  {
  std::string str, &i_str = i_list.front();
  
  for (unsigned i = 0; i < i_str.length(); ++i)
    str += toupper(i_str[i]);
  
  for (int i = 0; mg_commands[i].m_name != NULL; ++i)
    {
    if (str == mg_commands[i].m_name)
      {
      (this ->* mg_commands[i].m_function)(i_list);
      return;
      } 
    }
  
  m_socket << mg_err500;
  }
//------------------------------------------------------------------------------
void ServerThread::DoDialog()
  {
  char hostname[1024];

  if (gethostname(hostname, 1024))
    strcpy(hostname, "unknown-host");

  std::stringstream str;
  str << "<" << (long)time(NULL) << clock() << rand() << "@" << hostname << ">";
  m_msg_id = str.str();

  std::string banner = "220 ";
  banner += hostname;
  banner += " DictD++ " VERSION_STR " DICT server <mime.mime-type.auth> ";
  banner += m_msg_id;
  banner += "\r\n";
  
  m_socket << banner;
	
  std::string buf, quoted_string;
  std::vector<std::string> command_list;

  while(true)
    {
    if (!m_socket.IsValid())
      return;

    // Do we need new command from the client -
    // or do we have smth to do from the previous read?
    if (quoted_string.find("\r\n") == std::string::npos)
      {
      m_socket.FlushOutput();
      long timeout = m_server.GetTimeAccuracy();

      int res;

      while (!(res = m_socket.Select(Socket::SELECT_READ | Socket::SELECT_ERROR, timeout)))
        if (m_server.GetControlThread().IsShuttingDown())
          goto shutdown_check;

      if (res == SOCKET_ERROR || (res & Socket::SELECT_ERROR))
        {
        g_log << TRACE << "Socket error, stopping session with client" << endlog;
        m_socket.Close();
        return;
        }

      g_log << DEBUG << "Reading client input" << endlog;
      m_socket >> buf;
      g_log << DEBUG << "Command from client: " << buf << endlog;
      quoted_string += buf;
      }

shutdown_check:
    if (m_server.GetControlThread().IsShuttingDown())
      {
      g_log << INFO << "Shutdown has been requested, stopping session with client" << endlog;
      m_socket << "421 Server is shutting down at operator request\r\n";
      m_socket << "221 Bye\r\n";
      m_socket.FlushOutput();
      m_socket.Close();

      return;
      }

    std::string::size_type pos = quoted_string.find("\r\n");
    
    if (pos != std::string::npos)
      {
      std::string to_dequote = quoted_string.substr(0, pos);
      quoted_string = quoted_string.substr(pos + 2);
      
      if (!Tools::DequoteArgs(command_list, to_dequote))
        {
        g_log << ERR << "Error dequoting command \"" << quoted_string << "\" from client" << endlog;
	      m_socket << mg_err500;
	      continue;
        }
      
      int i = 0;
      for (std::vector<std::string>::iterator iter = command_list.begin();
           iter != command_list.end(); ++iter, ++i)
        g_log << TRACE << "Command[" << i << "] = " << *iter << endlog;
      
      if (command_list.empty())
        g_log << WARN << "Empty command from client has been detected" << endlog;
      else
        ProcessCommand(command_list);
      } // if (find)
    } // while (true)
  }
//------------------------------------------------------------------------------
void ServerThread::operator () ()
  {
    {
    boost::mutex::scoped_lock lock(m_server.m_mutex);

    g_log << TRACE << "New thread was started, number of running threads is " << (ulong)m_server.m_threads.size() << endlog;
    }

  UErrorCode status = U_ZERO_ERROR;
  mp_converter = ucnv_open("UTF-8", &status);
  assert(!U_FAILURE(status)); // Well, if we are not able even to create
                              // convertor from UTF-8 - I have no idea
                              // what is going with the program ...

  try
    {
    DoDialog();
    }
  catch(const SocketException& ex)
    {
    g_log << TRACE << ex << endlog;
    g_log << TRACE << "Socket exception, terminating thread" << endlog;
    }
  catch(const GeneralException& ex)
    {
    g_log << ERR << ex << endlog;
    g_log << ERR << "Cannot handle this exception on session level, terminating thread" << endlog;
    }
//  catch(...)
//    {
//    g_log << CRIT << "Unknown exception, terminating thread" << endlog;
//    }

  ucnv_close(mp_converter);

  boost::mutex::scoped_lock lock(m_server.m_mutex);
  g_log << TRACE << "Thread is being terminated, number of running threads is becoming " << (ulong)m_server.m_threads.size() - 1 << endlog;
  Server::ThreadContainer::iterator it = m_server.m_threads.find(this);
  assert(it != m_server.m_threads.end());
  delete it -> second;
  m_server.m_threads.erase(it);

  delete this;
  }
//------------------------------------------------------------------------------
