/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#include "stdafx.h"
#include "server_thread.h"
#include "server.h"

#include <dictionary_list.h>
#include <abstract_strategy.h>
#include <dictd_utils.h>

//------------------------------------------------------------------------------
const char* ServerThread::mg_info113 = "113 help text follows\r\n";
const char* ServerThread::mg_info230 = "230 Authentication successful\r\n";
const char* ServerThread::mg_info250 = "250 OK\r\n";
const char* ServerThread::mg_err500  = "500 Syntax error, command was not recognized\r\n";
const char* ServerThread::mg_err501  = "501 Syntax error, illegal parameters\r\n";
const char* ServerThread::mg_err502  = "502 Command is not implemented\r\n";
const char* ServerThread::mg_err530  = "530 Access denied, this user is not allowed to connect from your IP address\r\n";
const char* ServerThread::mg_err531  = "531 Access denied, use \"SHOW SERVER\" for server information\r\n";
const char* ServerThread::mg_err550  = "550 Invalid database, use \"SHOW DB\" for the list of databases\r\n";
const char* ServerThread::mg_err551  = "551 Invalid strategy, use \"SHOW STRAT\" for the list of strategies\r\n";
const char* ServerThread::mg_err552  = "552 No match\r\n";
const char* ServerThread::mg_err554  = "554 No databases present\r\n";
//------------------------------------------------------------------------------
ServerThread::CommandInfo ServerThread::mg_commands[] =
  {
    {"DEFINE", &ServerThread::CommDefine},
    {"MATCH", &ServerThread::CommMatch},
    {"SHOW", &ServerThread::CommShow},
    {"CLIENT", &ServerThread::CommClient},
    {"STATUS", &ServerThread::CommStatus},
    {"HELP", &ServerThread::CommHelp},
    {"QUIT", &ServerThread::CommQuit},
    {"OPTION", &ServerThread::CommOption},
    {"AUTH", &ServerThread::CommAuth},
    {"SASLAUTH", &ServerThread::CommSaslAuth},
    {NULL, NULL}
  };
//------------------------------------------------------------------------------
void ServerThread::CommDefine(const std::vector<std::string>& i_list)
  {
  if (i_list.size() != 3)
    {
    m_socket << mg_err501;
    return;
    }

  if (!m_server.GetDictionaryList().GetNumberOfAvailableDictionaries(m_session_info))
    {
    m_socket << mg_err554;
    return;
    }

  UErrorCode status = U_ZERO_ERROR;
  UnicodeString db_name(i_list[1].c_str(), (int32_t)i_list[1].size(), mp_converter, status);

  assert(!U_FAILURE(status));

  if (!m_server.GetDictionaryList().IsDictionaryPresent(db_name, m_session_info))
    {
    m_socket << mg_err550;
    return;
    }

  UnicodeString keyword_non_norm(i_list[2].c_str(), (int32_t)i_list[2].size(), mp_converter, status);
  assert(!U_FAILURE(status));

  UnicodeString keyword;
  Normalizer::normalize(keyword_non_norm, UNORM_NFD, 0, keyword, status);
  assert(!U_FAILURE(status));

  std::auto_ptr<AbstractController> p_controller(m_server.GetSecurityManager().CreateController(m_session_info));

  DictionaryList::DefinitionsQueryResult res;
  DictionaryList::QueryStatus query_status = m_server.GetDictionaryList().FindDefinitions(res, *p_controller, keyword, "exact", db_name, m_session_info);

  if (res.empty())
    {
    m_socket << mg_err552;
    return;
    }

  ulong num = 0;

  for (DictionaryList::DefinitionsQueryResult::const_iterator it = res.begin(); it != res.end(); ++it)
    num += (ulong)it -> second.size();

  m_socket << "150 " << boost::lexical_cast<std::string>(num) << " definitions found";

  if (query_status == DictionaryList::PARTIAL_RESULT)
    m_socket << " (search was interrupted, result is partial!)";

  m_socket << " - list follows\r\n";

  for (DictionaryList::DefinitionsQueryResult::const_iterator it = res.begin(); it != res.end(); ++it)
    for (Dictionary::Definitions::const_iterator it1 = it->second.begin(); it1 != it->second.end(); ++it1)
      {
      m_socket << "151 " << Tools::QuoteString(it1 -> first) << " " << Tools::QuoteString(it -> first -> GetDictionaryName()) << " " << Tools::QuoteString(it -> first -> GetDictionaryDescription()) << " - text follows\r\n";

      m_socket << it1 -> second;
      m_socket.UnquotedSend("\r\n.\r\n", 5);
      }

  m_socket << mg_info250;
  }
//------------------------------------------------------------------------------
void ServerThread::CommMatch(const std::vector<std::string>& i_list)
  {
  if (i_list.size() != 4)
    {
    m_socket << mg_err501;
    return;
    }

  if (!m_server.GetDictionaryList().GetNumberOfAvailableDictionaries(m_session_info))
    {
    m_socket << mg_err554;
    return;
    }

  UErrorCode status = U_ZERO_ERROR;
  UnicodeString strat_name(i_list[2].c_str(), (int32_t)i_list[2].size(), mp_converter, status);

  assert(!U_FAILURE(status));

  if (!m_server.GetDictionaryList().IsStrategyPresent(strat_name, m_session_info))
    {
    m_socket << mg_err551;
    return;
    }
  
  UnicodeString db_name(i_list[1].c_str(), (int32_t)i_list[1].size(), mp_converter, status);

  if (!m_server.GetDictionaryList().IsDictionaryPresent(db_name, m_session_info))
    {
    m_socket << mg_err550;
    return;
    }

  UnicodeString keyword_non_norm(i_list[3].c_str(), (int32_t)i_list[3].size(), mp_converter, status);
  assert(!U_FAILURE(status));

  UnicodeString keyword;
  Normalizer::normalize(keyword_non_norm, UNORM_NFD, 0, keyword, status);
  assert(!U_FAILURE(status));

  std::auto_ptr<AbstractController> p_controller(m_server.GetSecurityManager().CreateController(m_session_info));

  DictionaryList::KeyWordsQueryResult res;
  DictionaryList::QueryStatus query_status = m_server.GetDictionaryList().FindKeyWords(res, *p_controller, keyword, strat_name, db_name, m_session_info);

  if (res.empty())
    {
    m_socket << mg_err552;
    return;
    }

  ulong num = 0;

  for (DictionaryList::KeyWordsQueryResult::const_iterator it = res.begin(); it != res.end(); ++it)
    num += (ulong)it -> second.size();

  m_socket << "152 " << boost::lexical_cast<std::string>(num) << " matches found";

  if (query_status == DictionaryList::PARTIAL_RESULT)
    m_socket << " (search was interrupted, result is partial!)";

  m_socket << " - list follows\r\n";

  if (m_session_info.m_fl_use_mime)
    {
    m_socket << "Content-type: text/plain; charset=utf-8\r\n";
    m_socket << "Content-transfer-encoding: 8bit\r\n\r\n";
    }

  for (DictionaryList::KeyWordsQueryResult::const_iterator it = res.begin(); it != res.end(); ++it)
    for (std::list<UnicodeString>::const_iterator it1 = it->second.begin(); it1 != it->second.end(); ++it1)
      m_socket << Tools::QuoteString(it -> first -> GetDictionaryName()) << " " << Tools::QuoteString(*it1) << "\r\n";

  m_socket.UnquotedSend(".\r\n", 3);
  m_socket << mg_info250;
  }
//------------------------------------------------------------------------------
void ServerThread::CommShow(const std::vector<std::string>& i_list)
  {
  if (i_list.size() < 2)
    {
    m_socket << mg_err501;
    return;
    }

  std::string name;

  for (unsigned i = 0; i < i_list[1].length(); ++i)
    name += toupper(i_list[1][i]);

  if (name == "DB" || name == "DATABASES")
    {
    if (i_list.size() != 2)
      {
      m_socket << mg_err501;
      return;
      }

    const DictionaryList::DictionaryContainer& dict_cont = m_server.GetDictionaryList().GetDictionaryContainer();
    const DictionaryList::DictionarySetContainer& dict_set_cont = m_server.GetDictionaryList().GetDictionarySetContainer();

    ulong num = m_server.GetDictionaryList().GetNumberOfAvailableDictionaries(m_session_info);
    if (!num)
      {
      m_socket << mg_err554;
      return;
      }

    m_socket << "110 " << boost::lexical_cast<std::string>(num)
                       << " databases present - text follows\r\n";

    if (m_session_info.m_fl_use_mime)
      {
      m_socket << "Content-type: text/plain; charset=utf-8\r\n";
      m_socket << "Content-transfer-encoding: 8bit\r\n\r\n";
      }
    
    for (DictionaryList::DictionaryContainer::const_iterator it = dict_cont.begin();
         it != dict_cont.end(); ++it)
      if (m_server.GetDictionaryList().IsDictionaryAllowed(**it, m_session_info))
        m_socket << Tools::QuoteString((*it) -> GetDictionaryName()) << " " << Tools::QuoteString((*it) -> GetDictionaryDescription()) << "\r\n";

    for (DictionaryList::DictionarySetContainer::const_iterator it = dict_set_cont.begin();
         it != dict_set_cont.end(); ++it)
      if (m_server.GetDictionaryList().IsDictionarySetAllowed(it -> second, m_session_info))
        m_socket << Tools::QuoteString(it -> first) << " " << Tools::QuoteString(it -> second.first) << "\r\n";

    m_socket.UnquotedSend(".\r\n", 3);
    m_socket << mg_info250;
    }
  else if (name == "STRAT" || name == "STRATEGIES")
    {
    if (i_list.size() != 2)
      {
      m_socket << mg_err501;
      return;
      }

    const DictionaryList::StrategyContainer& strat_cont = m_server.GetDictionaryList().GetStrategyContainer();

    ulong num = 0;

    for (DictionaryList::StrategyContainer::const_iterator it = strat_cont.begin();
         it != strat_cont.end(); ++it)
      if (m_server.GetDictionaryList().IsStrategyPresent(it -> second.first -> GetStrategyName(), m_session_info))
        ++num;

    if (!num)
      {
      m_socket << "555 No strategies available";
      return;
      }

    m_socket << "111 " << boost::lexical_cast<std::string>(num)
                       << " strategies available - text follows\r\n";
    
    if (m_session_info.m_fl_use_mime)
      {
      m_socket << "Content-type: text/plain; charset=utf-8\r\n";
      m_socket << "Content-transfer-encoding: 8bit\r\n\r\n";
      }

    for (DictionaryList::StrategyContainer::const_iterator it = strat_cont.begin();
         it != strat_cont.end(); ++it)
      if (m_server.GetDictionaryList().IsStrategyPresent(it -> second.first -> GetStrategyName(), m_session_info))
        m_socket << Tools::QuoteString(it -> second.first -> GetStrategyName()) << " " << Tools::QuoteString(it -> second.first -> GetStrategyDescription()) << "\r\n";

    m_socket.UnquotedSend(".\r\n", 3);
    m_socket << mg_info250;
    }
  else if (name == "INFO")
    {
    if (i_list.size() != 3)
      {
      m_socket << mg_err501;
      return;
      }

    const DictionaryList::DictionaryContainer& dict_cont = m_server.GetDictionaryList().GetDictionaryContainer();
    const DictionaryList::DictionarySetContainer& dict_set_cont = m_server.GetDictionaryList().GetDictionarySetContainer();

    ulong num = m_server.GetDictionaryList().GetNumberOfAvailableDictionaries(m_session_info);

    if (!num)
      {
      m_socket << mg_err554;
      return;
      }

    UErrorCode status = U_ZERO_ERROR;
    UnicodeString db_name(i_list[2].c_str(), (int32_t)i_list[2].size(), mp_converter, status);

    assert(!U_FAILURE(status));

    for (DictionaryList::DictionaryContainer::const_iterator it = dict_cont.begin();
         it != dict_cont.end(); ++it)
      if ((*it) -> GetDictionaryName() == db_name && m_server.GetDictionaryList().IsDictionaryAllowed(**it, m_session_info))
        {
        m_socket << "112 database information follows\r\n";

        std::auto_ptr<AbstractController> p_controller(m_server.GetSecurityManager().CreateController(m_session_info));

        DictionaryList::DefinitionsQueryResult res;
        m_server.GetDictionaryList().FindDefinitions(res, *p_controller, "00-database-info", "exact",
                                                     (*it) -> GetDictionaryName(), m_session_info);
        assert(res.size() <= 1);

        if (!res.empty())
          m_socket << res.front().second.front().second; // Mmm, I like it ;-)
        else
          {
          if (m_session_info.m_fl_use_mime)
            {
            m_socket << "Content-type: text/plain; charset=utf-8\r\n";
            m_socket << "Content-transfer-encoding: 8bit\r\n\r\n";
            }

            m_socket << (*it) -> GetDictionaryDescription();
          }

        m_socket.UnquotedSend("\r\n.\r\n", 5);
        m_socket << mg_info250;
        return;
        }

    for (DictionaryList::DictionarySetContainer::const_iterator it = dict_set_cont.begin();
         it != dict_set_cont.end(); ++it)
      if (it -> first == db_name && m_server.GetDictionaryList().IsDictionarySetAllowed(it -> second, m_session_info))
        {
        m_socket << "112 database information follows\r\n";

        if (m_session_info.m_fl_use_mime)
          {
          m_socket << "Content-type: text/plain; charset=utf-8\r\n";
          m_socket << "Content-transfer-encoding: 8bit\r\n\r\n";
          }

        m_socket << it -> second.first;

        m_socket.UnquotedSend("\r\n.\r\n", 5);
        m_socket << mg_info250;
        return;
        }

    m_socket << mg_err550;
    }
  else if (name == "SERVER")
    {
    if (i_list.size() != 2)
      {
      m_socket << mg_err501;
      return;
      }

    m_socket << "114 server information follows\r\n";

    if (m_session_info.m_fl_use_mime)
      {
      m_socket << "Content-type: text/plain; charset=utf-8\r\n";
      m_socket << "Content-transfer-encoding: 8bit\r\n\r\n";
      }

    m_socket << m_server.m_server_info;
    m_socket.UnquotedSend("\r\n.\r\n", 5);
    m_socket << mg_info250;
    }
  else
    m_socket << mg_err500;

  }
//------------------------------------------------------------------------------
void ServerThread::CommClient(const std::vector<std::string>& i_list)
  {
  std::string str;

  for (std::vector<std::string>::const_iterator it = ++i_list.begin();
       it != i_list.end(); ++it)
    str += std::string(" ") + *it;

  g_log << INFO << "Client is" << str << endlog;

  m_socket << mg_info250;
  }
//------------------------------------------------------------------------------
void ServerThread::CommStatus(const std::vector<std::string>& i_list)
  {
  if (i_list.size() != 1)
    {
    m_socket << mg_err501;
    return;
    }

  m_socket << "210 " << m_server.GetStatistics() << "\r\n";
  }
//------------------------------------------------------------------------------
void ServerThread::CommHelp(const std::vector<std::string>& i_list)
  {
  if (i_list.size() != 1)
    {
    m_socket << mg_err501;
    return;
    }

  m_socket << mg_info113;

  if (m_session_info.m_fl_use_mime)
    {
    m_socket << "Content-type: text/plain; charset=utf-8\r\n";
    m_socket << "Content-transfer-encoding: 8bit\r\n\r\n";
    }
  
  m_socket <<
      "DEFINE database word         -- look up word in database\r\n"
	    "MATCH database strategy word -- match word in database using strategy\r\n"
	    "SHOW DB                      -- list all accessible databases\r\n"
	    "SHOW DATABASES               -- list all accessible databases\r\n"
	    "SHOW STRAT                   -- list available matching strategies\r\n"
	    "SHOW STRATEGIES              -- list available matching strategies\r\n"
	    "SHOW INFO database           -- provide information about the database\r\n"
	    "SHOW SERVER                  -- provide site-specific information\r\n"
      "OPTION MIME [mime-type]      -- output definitions in the requested MIME type\r\n"
	    "CLIENT info                  -- identify client to server\r\n"
	    "AUTH user string             -- provide authentication information\r\n"
	    "STATUS                       -- display timing information\r\n"
	    "HELP                         -- display this help information\r\n"
	    "QUIT                         -- terminate connection\r\n";

  m_socket << mg_info250;
  }
//------------------------------------------------------------------------------
void ServerThread::CommQuit(const std::vector<std::string>& i_list)
  {
  if (i_list.size() != 1)
    {
    m_socket << mg_err501;
    return;
    }

  m_socket << "221 Bye\r\n";
  m_socket.FlushOutput();
  m_socket.Close();
  }
//------------------------------------------------------------------------------
void ServerThread::CommOption(const std::vector<std::string>& i_list)
  {
  if (i_list.size() < 2 || i_list.size() > 3)
    {
    m_socket << mg_err501;
    return;
    }

  std::string name;

  for (unsigned i = 0; i < i_list[1].length(); ++i)
    name += toupper(i_list[1][i]);

  if (name == "MIME")
    {
    std::list<MimeType> mime_types_list;
    MimeType mime_type;

    if (i_list.size() == 3)
      {
      // We should parse requested mime types list:
      std::string cur_type;

      for (std::string::size_type i = 0; i < i_list[2].size(); ++i)
        if (i_list[2][i] == ',')
          {
          if (!mime_type.Construct(cur_type))
            {
            m_socket << mg_err501;
            return;
            }
          else
            {
            mime_types_list.push_back(mime_type);
            cur_type.clear();
            }
          }
        else
          cur_type += i_list[2][i];

      if (!mime_type.Construct(cur_type))
        {
        m_socket << mg_err501;
        return;
        }

      mime_types_list.push_back(cur_type);
      }
    else
      mime_types_list.push_back(MimeType("*/*"));
     
    mime_types_list.sort();

    m_session_info.m_fl_use_mime = true;
    m_session_info.m_mime_types = mime_types_list;

    m_socket << "250 OK, accepted output MIME types from now on are: \"";
    
    for (std::list<MimeType>::const_iterator it = mime_types_list.begin();
         it != mime_types_list.end(); ++it)
      {
      m_socket << it -> AsString(true);
      if (it != --mime_types_list.end())
        m_socket << ", ";
      }

    m_socket << "\"\r\n";
    }
  else
    m_socket << mg_err501;
  }
//------------------------------------------------------------------------------
void ServerThread::CommAuth(const std::vector<std::string>& i_list)
  {
  if (i_list.size() != 3)
    {
    m_socket << mg_err501;
    return;
    }

  const std::string& user = i_list[1], key = i_list[2];

  if (!m_server.GetSecurityManager().IsAuthenticated(m_msg_id, user, key))
    {
    m_socket << mg_err531;
    return;
    }

  if (!m_server.GetSecurityManager().IsConnectionAllowed(m_socket, user))
    {
    m_socket << mg_err530;
    return;
    }

  m_session_info.m_user = user;
  m_session_info.m_group = m_server.GetSecurityManager().GetUserGroup(user);

  g_log << TRACE << m_session_info.m_user << ":" << m_session_info.m_group << " authenticated" << endlog;

  m_socket << mg_info230;
  }
//------------------------------------------------------------------------------
void ServerThread::CommSaslAuth(const std::vector<std::string>& i_list)
  {
  m_socket << mg_err502;
  }
//------------------------------------------------------------------------------
