/*****************************************************************************
 *
 * DictD++, Copyright (C) 2002-2003 by Alexander Tsvyashchenko
 * contact address: <spam@ndl.kiev.ua>
 *
 * short license info is below, but it concerns only program itself -
 * you should see copyrights.txt for algorithms licenses info!
 *
 *****************************************************************************

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

******************************************************************************/

#ifndef SOCKET_H
#define SOCKET_H

#ifdef _WIN32
#include <winsock2.h>
#endif

#ifndef _WIN32
#define INVALID_SOCKET -1
#define SOCKET_ERROR -1
#endif

//--------------------------------------------------------------------
// This is an exception class used to report errors
// for all socket operations.
class SocketException: public GeneralException
  {
  public:
    SocketException(const std::string& i_err_id,
                    const std::string& i_msg):
      GeneralException("Network", i_err_id, i_msg) {}
  };
//--------------------------------------------------------------------
class Socket
  {
  public:
    enum Events
      {
      SELECT_READ  = 0x01,
      SELECT_WRITE = 0x02,
      SELECT_ERROR = 0x04,
      SELECT_ALL   = 0x07
      };

  public:
    Socket();
    virtual ~Socket();

    Socket(const Socket& i_socket);

    // Server initialization
    void Create();
    void Bind(unsigned port);
    void Listen() const;
    void Accept(Socket& o_socket) const;

    // Client initialization
    void Connect(const std::string& host, unsigned port);

    // Data Transimission
    void Send(const char* i_str, ulong i_len = 0);
    void Send(const std::string& i_str);
    void UnquotedSend(const char* ip_str, ulong len);
    void FlushOutput();
    void Receive( std::string& o_str);

    void SetNonBlocking(bool i_fl_non_blocking);
    bool IsNonBlocking() const { return m_fl_non_blocking; }

    unsigned long GetUnreadDataSize() const;

    void SetInputBufferSize(unsigned long i_in_buffer_size) { m_in_buffer.resize(i_in_buffer_size); }

    void SetOutputBufferSize(unsigned long i_out_buffer_size) { m_out_buffer.resize(i_out_buffer_size); }

    bool IsValid() const;

    int GetErrorCode() const;

    void Close();

    int Select(ulong i_events, long i_timeout) const;

    // Helper functions for easier access:
    Socket& operator << (const char* ip_str);

    Socket& operator << (const std::string& i_str);

    Socket& operator << (const UnicodeString& i_str);

    Socket& operator >> (std::string& o_str);

    // Informational functions:
    uint32_t GetPeerAddress() const;

  private:
#ifdef _WIN32
    SOCKET m_sock;
#else
    int m_sock;
#endif
    std::vector<char> m_in_buffer, m_out_buffer;
    ulong m_out_buffer_pos;
    bool m_fl_non_blocking;
    mutable bool m_fl_responsible;
  };
//--------------------------------------------------------------------

#endif // SOCKET_H
